#!/usr/bin/env python
# encoding=utf-8

##    Qemulator 0.4.4 - a qemu gui written in python and GTK/Glade.
##    Copyright (C) 2006  rainer haage
##
##    This program is free software; you can redistribute it and/or
##    modify it under the terms of the GNU General Public License
##    as published by the Free Software Foundation; either version 2
##    of the License, or (at your option) any later version.
##    
##    This program is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##    
##    You should have received a copy of the GNU General Public License
##    along with this program; if not, write to the Free Software
##    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import sys
import os
from os import path
import popen2
from popen2 import Popen3
import gobject
import gettext
import string
import pickle
import time
import threading
from threading import Thread
import shutil
import webbrowser
try:
    from egg import trayicon
    nognomextras = False
except :
    nognomextras = True
#nognomextras = True


import re
from copy import copy
from copy import deepcopy

try:
    import gettext
    import locale
except:
    print "locale and/or gettext for language support not installed"
    sys.exit(1)


try:
    import pygtk
    pygtk.require("2.0")
    
except:
      print "pygtk 2.0 is not installed"
try:
    import gtk
    import gtk.glade
    gtk.gdk.threads_init()
    
except:
    print "gtk and/or glade is not installed - exiting..."
    sys.exit(1)
    
encoding = locale.getpreferredencoding()
utf8conv = lambda x : unicode(x, encoding).encode('utf8')
    
#for param in os.environ.keys():
#    print "%20s %s" % (param,os.environ[param])
    
current_path = os.path.realpath(__file__)
basedir = os.path.dirname(os.path.realpath(__file__))
if not os.path.exists(os.path.join(basedir, "main.py")):
    if os.path.exists(os.path.join(os.getcwd(), "main.py")):
        basedir = os.getcwd()
sys.path.insert(0, basedir)
os.chdir(basedir)  
#print "basedir: " + basedir
basedir = basedir.replace ( '/bin', '' ) 
basedir = basedir.replace ( '/lib/qemulator', '' ) 
sys.path.append(basedir)   
bindir = basedir + "/bin"
#print "bindir: " + bindir
libdir = basedir + "/lib/qemulator"
#print "libdir: " + libdir
sharedir = basedir + "/share/qemulator"
#print "sharedir: " + sharedir
pixmapdir = basedir + "/share/pixmaps/qemulator"
#print "pixmapdir: " + pixmapdir     
homedir = os.path.expanduser('~/.qemulator')
configdir = homedir + "/config"
#print "configdir: " + configdir           
icondirs = [sharedir + "/icons",homedir + "/icons"] 

## Import all Qemulator classes 
import qml_network
import qml_controlmon
import qml_ptsmanager
#import qml_runqemu
import qml_configuration     
import qml_filehandlers
import qml_style

APP = 'Qemulator'
#DIR = 'locale'
DIR = basedir + '/share/locale'
#print "DIR: " + DIR
locale.setlocale(locale.LC_ALL, '')
gettext.bindtextdomain(APP, DIR)
gettext.textdomain(APP)
_ = gettext.gettext

gtk.glade.bindtextdomain(APP, DIR)
gtk.glade.textdomain(APP)
tooltips = gtk.Tooltips() 

#print "home: " + os.environ["HOME"]

def storetofile(indata, filename):
    #print "storefile: " + filename
    dumpf = open(configdir + "/" + filename ,"w")        
    pickle.dump(indata, dumpf)
    dumpf.close()
    
## Universal Function to open the pickle'd configfiles        
def getfromfile(filename):
    try:
        abs_path = path.abspath(configdir + "/" + filename)  
        dumpf = open( abs_path ,"r")       
        outdata = pickle.load(dumpf)
        dumpf.close()
        return outdata        
    except:
        outdata  = { }
        print "File not found: " + str(filename)
        return outdata
    
class Splashscreen ( Thread ):

    def __init__ (self):
        Thread.__init__(self) 
        self.gladefile = libdir + "/qemulator.glade"

    def init_gui(self):
        logo_file = None
        logo_file = pixmapdir + "/splashscreen.png"
        
        #gtk.threads_enter()
        self.wTree_splash = gtk.glade.XML(self.gladefile, "window_splashscreen", APP)
        self.window_splashscreen = self.wTree_splash.get_widget("window_splashscreen")
        self.image_splashscreen = self.wTree_splash.get_widget("image_splashscreen")
        self.progressbar_splashscreen = self.wTree_splash.get_widget("progressbar_splashscreen")
        self.label_progress_splashscreen = self.wTree_splash.get_widget("label_progress_splashscreen")                
        
        self.image_splashscreen.set_from_file( logo_file)
        gobject.timeout_add(15000, self.splash_destroy) # 5*1000 miliseconds
        #time.sleep(0.1)
        self.window_splashscreen.show()
        #gtk.threads_leave()     
        
    def run(self): 
        
        gtk.threads_enter()
        self.init_gui()
        gtk.threads_leave()
        
    def increase_progress(self, amount):
        self.progress = self.progress + amount
        #print "progress: " + str(self.progress)
        self.progressbar_splashscreen.set_fraction( self.progress)

    def set_progress(self, amount):
        self.progress = amount
        #print "progress: " + str(self.progress)
        #gtk.threads_enter()
        self.progressbar_splashscreen.set_fraction(self.progress) 
        #gtk.threads_leave()
        
    def set_progresstext(self, text):
        #gtk.threads_enter()
        self.progressbar_splashscreen.set_text(text) 
        #gtk.threads_leave()        
    def splash_hide(self):
        #gtk.threads_enter()
        self.window_splashscreen.hide()
        #gtk.threads_leave()
    def splash_destroy(self):
        #gtk.threads_enter()
        self.window_splashscreen.destroy()
        #gtk.threads_leave()        

class Qemulator(Thread):

## Start main app ##################################
    def __init__(self, settings):
    #def __init__ (self,ip):
        Thread.__init__(self)
        self.currimage_settings = QemuConfig()
        self.basedir = basedir 
        #print "basedir: " + self.basedir 
        self.bindir = self.basedir + "/bin"
        self.libdir = self.basedir + "/lib/qemulator"
        self.sharedir = self.basedir + "/share/qemulator"
        self.pixmapdir = pixmapdir
        self.homedir = os.path.expanduser('~/.qemulator')
        self.configdir = self.homedir + "/config"           
        self.icondirs = [self.sharedir + "/icons",self.homedir + "/icons"]
        self.statefiles_dir = homedir + "/statefiles"
        self.screenshot_dir = homedir + "/screenshots"
        self.capture_dir = homedir + "/capture"
        self.gladefile = self.libdir + "/qemulator.glade"
        self.restoreaction = False
        self.currimage_settings.cdrmedium = ""
        self.imagespath = ""
        self.ilist_data = []
        self.default_machine_icon = self.icondirs[0] + "/default.png"
        self.currentimage = ""
        self.statefile = None
        self.joblist_data = []
        self.machinecommand = ""
        self.jobmessage = []
        self.dorun = True
        self.doexit = False
        self.block_jobmon_update = False
        self.jobmanager = None
        self.settings_autoshow = True
        self.settings_autohide = True
        self.default_iconsize = 32  
        self.jobmanagers = {}
        self.image_settings  = None

        ## Instanciate used helper classes
        self.settings = settings

        self.stateshandler = None
        self.sceenshothandler = None
        self.capturehandler = None      
            
    def run(self): 
        self.preload_gui()
        
        if splashscreen != None:
            gtk.threads_enter()
            self.update_splashscreen(0.2, _("Preloaded GUI"))
            gtk.threads_leave()
            time.sleep(0.2)
        
        self.image_settings  = QemuConfig()  
        self.jobwatcher = QemuWatcher(self.joblist.remove_job2)
        self.load_joblist()
        if splashscreen != None:
            gtk.threads_enter()
            self.update_splashscreen(0.3, _("Loaded joblist"))
            gtk.threads_leave()
            time.sleep(0.2)
            
        self.check_firstrun()
        
        if splashscreen != None:
            gtk.threads_enter()
            self.update_splashscreen(0.4, _("Check first run"))
            gtk.threads_leave()
            time.sleep(0.2)

        self.fetch_settings()
        
        if splashscreen != None:
            gtk.threads_enter()
            self.update_splashscreen(0.6, _("Loaded settings from file"))
            gtk.threads_leave()
            time.sleep(0.2)

        self.init_imagesettings()  
            
        if splashscreen != None:
            gtk.threads_enter()
            self.update_splashscreen(0.8, _("Loaded image settings from file"))
            gtk.threads_leave()
            time.sleep(0.2)

        self.load_bookmarks()
        
        if splashscreen != None:
            gtk.threads_enter()
            self.update_splashscreen(0.9, _("Loaded My machines from file"))
            gtk.threads_leave()
            time.sleep(0.2)

        self.init_gui()

        if splashscreen != None:
            gtk.threads_enter()
            self.update_splashscreen(1.0, _("Qemulator started"))
            gtk.threads_leave()
            time.sleep(0.3)
            #gtk.threads_enter()
            #splashscreen.splash_hide()
            splashscreen.splash_destroy()
            #gtk.threads_leave()
        else:
            print "NO SPLASH!"
        
    def init_joblist(self): 
        self.joblist.jobmanagers = self.jobmanagers
        self.joblist.load_joblist()
        self.joblist.buildtree_joblist()
        
    def init_imagesettings(self):
        try:
            self.image_settings = getfromfile("image_settings")
        except:
            print "no image settings found"        
        
        
    def update_splashscreen(self, value, text=None):
        #gtk.threads_enter()
        splashscreen.set_progress(value)
        if text != None:
            splashscreen.set_progresstext(text)
        #gtk.threads_leave()
        
    def preload_gui(self):
        
        ## Init GUI
        gtk.threads_enter()
        self.wTree = gtk.glade.XML(self.gladefile, "window1", APP)
        self.vpaned_mainwindow = self.wTree.get_widget("vpaned_mainwindow")
        self.mainwindow = self.wTree.get_widget("window1")
        self.viewport_settings_hardware = self.wTree.get_widget("viewport_settings_hardware")
        
        self.combobox_runmode = self.wTree.get_widget("combobox_runmode")
        self.toolbutton_launch = self.wTree.get_widget("toolbutton_launch")
        toolbutton_launch_style = qml_style.WidgetStyle(self.toolbutton_launch)
        toolbutton_launch_style.set_bg('#9AD08F')
        self.vbox_topane = self.wTree.get_widget("vbox_topane")
        self.vbox_bottompane = self.wTree.get_widget("vbox_bottompane")
        self.viewport_bottompane = self.wTree.get_widget("viewport_bottompane")
        
        self.logo_mainwindow = self.wTree.get_widget("logo_mainwindow")
        self.button_togglesettings = self.wTree.get_widget("button_togglesettings")
        self.checkbutton_autoshow_settings = self.wTree.get_widget("checkbutton_autoshow_settings")
        self.checkbutton_autohide_settings = self.wTree.get_widget("checkbutton_autohide_settings")
        
        self.menu_widgets_mainmenu = gtk.glade.XML(self.gladefile,"menubar1", APP)
        self.menu_mainmenu = self.menu_widgets_mainmenu.get_widget("menubar1")
        
        self.menu_widgets_settingsmenu = gtk.glade.XML(self.gladefile,"menuitem_edit_menu", APP)
        self.item_set_autohide = self.menu_widgets_settingsmenu.get_widget("item_set_autohide")
        self.item_set_autoshow = self.menu_widgets_settingsmenu.get_widget("item_set_autoshow")
        
        self.isolist = self.wTree.get_widget("treeview_bootimages")
        self.treeview_bookmarks = self.wTree.get_widget("treeview_bookmarks")
        self.treeview_joblist = self.wTree.get_widget("treeview_joblist")
        self.treeview_cdrives = self.wTree.get_widget("treeview_cdrives")
        
        self.notebook_imgsettings = self.wTree.get_widget("notebook_imgsettings")
        
        self.combobox_imagedevice = self.wTree.get_widget("combobox_imagedevice")
        self.combobox_cdromdrive = self.wTree.get_widget("combobox_cdromdrive")
        self.combobox_audiohw = self.wTree.get_widget("combobox_audiohw")
        self.combobox_emusystem = self.wTree.get_widget("combobox_emusystem")
        self.combobox_machinetype = self.wTree.get_widget("combobox_machinetype")
        
        self.filechooserbutton_imagecdrom = self.wTree.get_widget("filechooserbutton_imagecdrom")
        self.eventbox_imagecdrom = self.wTree.get_widget("eventbox_imagecdrom")
        
        self.filechooserbutton_hda = self.wTree.get_widget("filechooserbutton_hda")
        self.eventbox_filechooserbutton_hda = self.wTree.get_widget("eventbox_filechooserbutton_hda")
        
        self.filechooserbutton_hdb = self.wTree.get_widget("filechooserbutton_hdb")
        self.eventbox_filechooserbutton_hdb = self.wTree.get_widget("eventbox_filechooserbutton_hdb")
        
        self.filechooserbutton_hdc = self.wTree.get_widget("filechooserbutton_hdc")
        self.eventbox_filechooserbutton_hdc = self.wTree.get_widget("eventbox_filechooserbutton_hdc")
        
        self.filechooserbutton_hdd = self.wTree.get_widget("filechooserbutton_hdd")
        self.eventbox_filechooserbutton_hdd = self.wTree.get_widget("eventbox_filechooserbutton_hdd")
        
        self.filechooserbutton_fda = self.wTree.get_widget("filechooserbutton_fda")
        self.eventbox_filechooserbutton_fda = self.wTree.get_widget("eventbox_filechooserbutton_fda")
        
        self.filechooserbutton_fdb = self.wTree.get_widget("filechooserbutton_fdb") 
        self.eventbox_filechooserbutton_fdb = self.wTree.get_widget("eventbox_filechooserbutton_fdb")
        
        self.button_eject_imagecdrom = self.wTree.get_widget("button_eject_imagecdrom")
        self.button_eject_hda = self.wTree.get_widget("button_eject_hda")
        self.button_eject_hdb = self.wTree.get_widget("button_eject_hdb")
        self.button_eject_hdc = self.wTree.get_widget("button_eject_hdc")
        self.button_eject_hdd = self.wTree.get_widget("button_eject_hdd")
        self.button_eject_fda = self.wTree.get_widget("button_eject_fda")
        self.button_eject_fdb = self.wTree.get_widget("button_eject_fdb")     
        
        self.radiobutton_cdromtype = self.wTree.get_widget("radiobutton_cdromtype")
        self.radiobutton_cdromtype1 = self.wTree.get_widget("radiobutton_cdromtype1")
        self.radiobutton_cdromtype2 = self.wTree.get_widget("radiobutton_cdromtype2")
        
        self.radiobutton_nokqemu = self.wTree.get_widget("radiobutton_nokqemu")
        self.radiobutton_kqemu = self.wTree.get_widget("radiobutton_kqemu") 
        self.radiobutton_kkqemu = self.wTree.get_widget("radiobutton_kkqemu") 
        # Widgets for the boot options
        self.checkbutton_dlinuxboot = self.wTree.get_widget("checkbutton_dlinuxboot")
        self.frame_dlinuxboot = self.wTree.get_widget("frame_dlinuxboot")
        self.button_hide_dlinuxboot = self.wTree.get_widget("button_hide_dlinuxboot")
        self.filechooserbutton_kernelimage = self.wTree.get_widget("filechooserbutton_kernelimage")
        self.filechooserbutton_initrd = self.wTree.get_widget("filechooserbutton_initrd")
        self.entry_kernelcmd = self.wTree.get_widget("entry_kernelcmd")
        self.hbox_imagedevice = self.wTree.get_widget("hbox_imagedevice")
        self.checkbutton_snapshotmode = self.wTree.get_widget("checkbutton_snapshotmode")
        
        self.checkbutton_usevga = self.wTree.get_widget("checkbutton_usevga") 
        self.checkbutton_novga = self.wTree.get_widget("checkbutton_novga") 
        self.checkbutton_fullscreen = self.wTree.get_widget("checkbutton_fullscreen")
        self.checkbutton_startvnc = self.wTree.get_widget("checkbutton_startvnc")
        self.checkbutton_enable_usb = self.wTree.get_widget("checkbutton_enable_usb")
                
        self.checkbutton_localtime = self.wTree.get_widget("checkbutton_localtime")
        
        self.spinbutton_vncdisplay = self.wTree.get_widget("spinbutton_vncdisplay")
        self.spinbutton_memory = self.wTree.get_widget("spinbutton_memory")
        
        self.modulewindow = self.wTree.get_widget("vbox2")
        self.button_openimage_cancel = self.wTree.get_widget("button_openimage_cancel")
        self.entry_current_image = self.wTree.get_widget("entry_current_image")
        
        self.entry_networkline = self.wTree.get_widget("entry_networkline")
        self.entry_additional_options = self.wTree.get_widget("entry_additional_options")
        
        self.toolbutton_add_bookmark = self.wTree.get_widget("toolbutton_add_bookmark")
        #self.toolbutton_remove_bookmark = self.wTree.get_widget("toolbutton_remove_bookmark")
        
        self.button_apply_imagesettings = self.wTree.get_widget("button_apply_imagesettings")
        self.button_restore_imagesettings = self.wTree.get_widget("button_restore_imagesettings")
        self.label_imgsettings = self.wTree.get_widget("label_imgsettings")
        self.menu_widgets_imagelist = gtk.glade.XML(self.gladefile,"menu_popup_imagelist", APP)
        self.menu_popup_imagelist = self.menu_widgets_imagelist.get_widget("menu_popup_imagelist")
        gtk.threads_leave()
        
        dic_menu_imagelist = {"on_item_info_activate" : self.show_infomessage
        , "on_item_bookmark_activate" : self.add_bookmark
        }
        self.menu_widgets_imagelist.signal_autoconnect(dic_menu_imagelist)
        
        gtk.threads_enter()
        self.menu_widgets_bookmarks = gtk.glade.XML(self.gladefile,"menu_popup_bookmarks", APP)
        self.menu_popup_bookmarks = self.menu_widgets_bookmarks.get_widget("menu_popup_bookmarks")
        self.item_show_settings = self.menu_widgets_bookmarks.get_widget("item_show_settings")
        self.item_hide_settings = self.menu_widgets_bookmarks.get_widget("item_hide_settings")
        gtk.threads_leave()
        dic_menu_bookmarks = {"on_bookmark_info_activate" : self.show_infomessage_b
        , "on_delete_bookmark_activate" : self.remove_bookmark
        , "on_edit_bookmark_activate" : self.add_bookmark
        , "on_item_show_settings_activate" : self.show_settings
        , "on_item_hide_settings_activate" : self.hide_settings
        }
        self.menu_widgets_bookmarks.signal_autoconnect(dic_menu_bookmarks)       
        gtk.threads_enter()
        self.menu_widgets_joblist = gtk.glade.XML(self.gladefile,"menu_popup_joblist", APP)
        self.menu_popup_joblist = self.menu_widgets_joblist.get_widget("menu_popup_joblist")
        self.item_stop_job = self.menu_widgets_joblist.get_widget("item_stop_job")
        self.item_cont_job = self.menu_widgets_joblist.get_widget("item_cont_job")
        self.item_reset_job = self.menu_widgets_joblist.get_widget("item_reset_job")
        self.item_kill_job = self.menu_widgets_joblist.get_widget("item_kill_job")
        gtk.threads_leave()
        dic_menu_joblist = {"on_item_jobmonoitor_activate" : self.run_jobmanager
        ,"on_item_stop_job_activate" : self.pause_job
        , "on_item_cont_job_activate" : self.cont_job
        #,"on_item_stop_job_activate" : self.jobmanagers[self.selected_job[1]].pause_job
        ,"on_item_reset_job_activate" : self.reset_job
        , "on_item_kill_job_activate" : self.stop_job        
        }
        self.menu_widgets_joblist.signal_autoconnect(dic_menu_joblist)      
        
        gtk.threads_enter()
        self.wTreePopupScreenshots = gtk.glade.XML(self.gladefile,"menu_popup_screenshots", APP)
        self.menu_popup_screenshots = self.wTreePopupScreenshots.get_widget("menu_popup_screenshots")
        self.item_use_as_icon = self.wTreePopupScreenshots.get_widget("item_use_as_icon")
        gtk.threads_leave()
        
        dic_menu_popup_screenshots = {"on_item_use_as_icon_activate" : self.scr2icon
        }
        self.wTreePopupScreenshots.signal_autoconnect(dic_menu_popup_screenshots)                

        # Network widgets
        gtk.threads_enter()
        
        
        
        self.frame_network_usermode = self.wTree.get_widget("frame_network_usermode")
        self.frame_network_custom = self.wTree.get_widget("frame_network_custom")
        
        self.radiobutton_network_nonet = self.wTree.get_widget("radiobutton_network_nonet")
        self.radiobutton_network_usermode = self.wTree.get_widget("radiobutton_network_usermode")
        self.radiobutton_network_customsetup = self.wTree.get_widget("radiobutton_network_customsetup")
        
        self.toolbutton_network_netcard_add = self.wTree.get_widget("toolbutton_network_netcard_add")
        self.toolbutton_network_redir_add = self.wTree.get_widget("toolbutton_network_redir_add")
        #self.button_network_edit = self.wTree.get_widget("button_network_edit")
        
        self.toolbutton_network_netcard_edit = self.wTree.get_widget("toolbutton_network_netcard_edit")
        self.toolbutton_network_redir_edit = self.wTree.get_widget("toolbutton_network_redir_edit")
        
        #self.button_network_remove = self.wTree.get_widget("button_network_remove")
        
        self.toolbutton_network_netcard_delete = self.wTree.get_widget("toolbutton_network_netcard_delete")
        self.toolbutton_network_redir_delete = self.wTree.get_widget("toolbutton_network_redir_delete")
        
        self.treeview_networkcards = self.wTree.get_widget("treeview_networkcards")
        self.treeview_redirects = self.wTree.get_widget("treeview_redirects")
        self.label_network_netcards = self.wTree.get_widget("label_network_netcards")
        self.label__network_redirects = self.wTree.get_widget("label__network_redirects")
        self.filechooserbutton_network_smbshare = self.wTree.get_widget("filechooserbutton_network_smbshare")
        self.checkbutton_smbshare = self.wTree.get_widget("checkbutton_smbshare")
        self.entry_network_hostname = self.wTree.get_widget("entry_network_hostname")
        self.hbox_usernet_vlan = self.wTree.get_widget("hbox_usernet_vlan")
        self.label_hostname = self.wTree.get_widget("label_hostname")
        self.spinbutton_usernet_vlan = self.wTree.get_widget("spinbutton_usernet_vlan")
        
        self.treeview_statefiles = self.wTree.get_widget("treeview_statefiles")
        
        self.iconview_screenshots = self.wTree.get_widget("iconview_screenshots")
        
        self.treeview_capture = self.wTree.get_widget("treeview_capture")
        
        
        self.joblist = Joblist(self.treeview_joblist, self.joblist_data)  
        #self.jobwatcher = QemuWatcher(self.joblist.remove_job2) 
        gtk.threads_leave()
        #deletesig = self.wTree.signal_connect("on_window1_delete_event", self.delete_event)
        #print "handler deletesig: " + str(deletesig)
        self.sighandlers = {}
        
        self.imageselection = self.entry_current_image.connect("changed",  self.select_image_from_widget)
        #print "handler imageselection: " + str(self.imageselection)
        self.sighandlers['filechooserbutton_imagecdrom'] = self.filechooserbutton_imagecdrom.connect("selection_changed",  self.load_cdromimage)
        
        
        
        self.sighandlers['filechooserbutton_hda'] = self.filechooserbutton_hda.connect("selection_changed",  self.load_xtra_image)
        self.sighandlers['filechooserbutton_hdb'] = self.filechooserbutton_hdb.connect("selection_changed",  self.load_xtra_image)
        self.sighandlers['filechooserbutton_hdc'] = self.filechooserbutton_hdc.connect("selection_changed",  self.load_xtra_image)
        self.sighandlers['filechooserbutton_hdd'] = self.filechooserbutton_hdd.connect("selection_changed",  self.load_xtra_image)
        self.sighandlers['filechooserbutton_fda'] = self.filechooserbutton_fda.connect("selection_changed",  self.load_xtra_image)
        self.sighandlers['filechooserbutton_fdb'] = self.filechooserbutton_fdb.connect("selection_changed",  self.load_xtra_image)
        
        #, "on_filechooserbutton_imagecdrom_selection_changed" : self.load_cdromimage
        
        
        
        self.maindic = {"on_window1_delete_event" : self.delete_event
                ,"on_beenden1_activate" : self.exit_app_event
                
                , "on_item_about_activate" : self.init_about
                , "on_item_donate_activate" : self.dodonate
                
                , "on_item_create_image_activate" : self.init_create_image
                , "on_item_settings_activate" : self.open_settings
                , "on_item_homepage_activate" : self.open_website
                
                , "on_checkbutton_autohide_settings_toggled" : self.menu_toggle_settings_autohide
                , "on_checkbutton_autoshow_settings_toggled" : self.menu_toggle_settings_autoshow
                
                , "on_item_quit_activate" : self.exit_app_event
                
                , "on_toolbutton1_clicked" : self.get_soundhw
                , "on_toolbutton4_clicked" : self.open_settings
                , "on_toolbutton_launch_clicked" : self.run_qemu
                , "on_button_togglesettings_clicked" : self.toggle_settingsview
                
                , "on_treeview_bootimages_button_press_event" : self.pop_up_imagelist     
                , "on_treeview_bootimages_cursor_changed" : self.treeview_selection
                , "on_treeview_cdrives_cursor_changed" : self.cdrive_selection                     
                , "on_treeview_bookmarks_button_press_event" : self.pop_up_bookmarks
                , "on_treeview_bookmarks_cursor_changed" : self.bookmark_selection
                , "on_treeview_bookmarks_drag_end" : self.sort_bookmarks
                , "on_treeview_joblist_cursor_changed" : self.joblist_selection
                , "on_treeview_joblist_button_press_event" : self.pop_up_joblist

                , "on_button_apply_imagesettings_clicked" : self.store_emu_settings
                #, "on_button_restore_imagesettings_clicked": self.restore_emu_settings
                , "on_button_restore_imagesettings_clicked": self.restore_emu_config
                
                , "on_combobox_imagedevice_changed" : self.select_bootimage
                , "on_combobox_cdromdrive_changed" : self.select_cdromdrive
                #, "on_filechooserbutton_imagecdrom_selection_changed" : self.load_cdromimage

                , "on_combobox_audiohw_changed" : self.select_soundhw
                , "on_combobox_emusystem_changed" : self.select_emusystem
                , "on_combobox_machinetype_changed" : self.select_machine

                #, "on_filechooserbutton_hda_selection_changed" : self.load_xtra_image
                #, "on_filechooserbutton_hdb_selection_changed" : self.load_xtra_image
                #, "on_filechooserbutton_hdc_selection_changed" : self.load_xtra_image
                #, "on_filechooserbutton_hdd_selection_changed" : self.load_xtra_image
                #, "on_filechooserbutton_fda_selection_changed" : self.load_xtra_image
                #, "on_filechooserbutton_fdb_selection_changed" : self.load_xtra_image                              
                
                , "on_button_eject_imagecdrom_clicked" : self.clear_filechooser
                , "on_button_eject_hda_clicked" : self.clear_filechooser
                , "on_button_eject_hdb_clicked" : self.clear_filechooser
                , "on_button_eject_hdc_clicked" : self.clear_filechooser
                , "on_button_eject_hdd_clicked" : self.clear_filechooser
                , "on_button_eject_fda_clicked" : self.clear_filechooser
                , "on_button_eject_fdb_clicked" : self.clear_filechooser
                
                , "on_radiobutton_cdromtype_toggled" : self.no_xtracdrom
                , "on_radiobutton_cdromtype1_toggled" : self.select_mountcdrom
                , "on_radiobutton_cdromtype2_toggled" : self.select_imagecdrom
                
                , "on_radiobutton_nokqemu_toggled" : self.set_nokqemu
                , "on_radiobutton_kqemu_toggled" : self.set_kqemu
                , "on_radiobutton_kkqemu_toggled" : self.set_kkqemu
                
                , "on_checkbutton_usevga_toggled" : self.set_vgamode
                , "on_checkbutton_novga_toggled" : self.set_novga
                , "on_checkbutton_fullscreen_toggled" : self.set_fullscreen
                
                , "on_checkbutton_enable_usb_toggled" : self.set_usb
                
                , "on_checkbutton_dlinuxboot_toggled" : self.toggle_dlinuxboot
                , "on_button_hide_dlinuxboot_clicked" : self.toggle_hide_dlinuxboot
                
                , "on_filechooserbutton_kernelimage_selection_changed" : self.load_kernelimage
                , "on_filechooserbutton_initrd_selection_changed" : self.load_initrd
                , "on_entry_kernelcmd_changed" : self.set_kernelcmd
                
                , "on_checkbutton_snapshotmode_toggled" : self.set_snapshotmode
                , "on_checkbutton_localtime_toggled" : self.set_localtimemode
                
                , "on_checkbutton_startvnc_toggled" : self.set_startvnc
                , "on_spinbutton_vncdisplay_changed" : self.set_vncdisplay
                , "on_spinbutton_memory_changed" : self.set_memory
                
                , "on_button_load_image_clicked" : self.fileChooser_emuimage
                , "on_item_load_image_activate" : self.fileChooser_emuimage
                , "on_filechooserdialog_openimage_response" : self.fileChooser_emuimage_response
                
                , "on_toolbutton_add_bookmark_clicked" : self.add_bookmark
                #, "on_toolbutton_remove_bookmark_clicked" : self.remove_bookmark

                # handlers for the network widgets
                , "on_toolbutton_network_netcard_add_clicked"  : self.init_networksetup
                , "on_toolbutton_network_redir_add_clicked" : self.init_network_redirect
                #, "on_button_network_remove_clicked" : self.remove_netcard
                
                , "on_toolbutton_network_netcard_delete_clicked" : self.remove_netcard
                , "on_toolbutton_network_redir_delete_clicked" : self.remove_netcard
                
                #, "on_button_network_edit_clicked"  : self.init_networksetup
                
                , "on_toolbutton_network_netcard_edit_clicked"  : self.init_networksetup
                , "on_toolbutton_network_redir_edit_clicked"  : self.init_networksetup
                
                , "on_entry_additional_options_changed" : self.set_customoptions
                , "on_radiobutton_network_nonet_toggled" : self.toggle_networkuse
                , "on_radiobutton_network_usermode_toggled" : self.toggle_networkuse
                , "on_radiobutton_network_customsetup_toggled" : self.toggle_networkuse
                , "on_entry_network_hostname_changed" : self.set_usernet_hostname
                , "on_spinbutton_usernet_vlan_changed" : self.set_usernet_vlan
                , "on_treeview_networkcards_cursor_changed" : self.select_networkcard
                , "on_treeview_redirects_cursor_changed" : self.select_networkredir
                , "on_checkbutton_smbshare_toggled" : self.set_smbsettings
                , "on_filechooserbutton_network_smbshare_selection_changed" : self.set_smbsettings
                
                # handlers for the new image dialog
                , "on_filechooserbutton_newimage_dest_current_folder_changed" : self.set_newimage_folder
                , "on_entry_newimage_name_changed" : self.set_newimage_name
                , "on_combobox_newimage_format_changed" : self.set_newimageformat
                , "on_spinbutton_newimage_size_changed" : self.set_newimagesize
                , "on_combobox_newimage_sizeunit_changed" : self.set_newimagesize
                , "on_button_create_image_clicked" : self.create_newimage
                
                #, "on_button_savestate_clicked" : self.store_statefile
                , "on_toolbutton_runstate_clicked" : self.load_statefile_clicked
                , "on_toolbutton_delstate_clicked" : self.delete_statefile_clicked
                , "on_toolbutton_refresh_statefiles_clicked" : self.refresh_statefiles
                
                #, "on_toolbutton_storescreenshot_clicked" : self.store_screenshot
                , "on_iconview_screenshots_button_press_event"  : self.screenshot_clicked
                , "on_toolbutton_viewscreenshot_clicked"  : self.run_imageviewer
                , "on_toolbutton_deletescreenshot_clicked"  : self.delete_screenshot_clicked
                , "on_toolbutton_refresh_screenshots_clicked" : self.refresh_screenshots
                            
                #, "on_toggletoolbutton_record_toggled" : self.toggle_capture
                , "on_toolbutton_trashaudio_clicked" : self.delete_capture_clicked
                , "on_treeview_capture_button_press_event"  : self.run_audioplayer
                , "on_toolbutton_playaudio_clicked"  : self.run_audioplayer
                , "on_toolbutton_refresh_captures_clicked" : self.refresh_captures
                }
        self.wTree.signal_autoconnect(self.maindic)   

    ## initialisation of the widgets default settings and models ##
    def init_gui(self, *widget):
        
        
        logo_file = pixmapdir + "/icon3-32.png"
        icon_file = pixmapdir + "/icon3-16.png"
        
        gtk.threads_enter()
        self.mainwindow.set_icon_from_file( icon_file)
        self.logo_mainwindow.set_from_file( logo_file)
        self.switch_settingsview("hide")
        self.combobox_runmode.set_active(0)
        self.radiobutton_kqemu.set_active(True)
        self.combobox_imagedevice.set_active(1)
        self.select_bootimage(self.combobox_imagedevice)
        self.toggle_settings_autohide(self.settings_autohide)
        self.toggle_settings_autoshow(self.settings_autoshow)
        gtk.threads_leave()
        
        try:
            data=os.listdir(self.settings["imagespath"])
            data.sort()
        except:
            data = ["", _('Please go to settings and select a default images folder as the Home for your qemu-images'), ""]
           
        for entry in data:
            path = str(self.imagespath) + "/" + str(entry)
            haveconfig = self.check_haveconfig(path)
            self.ilist_data.append((path, haveconfig))  
                      
        gtk.threads_enter()    
        ilistmodel = self.create_model_ilist(self.ilist_data)      
        self.isolist.set_model( ilistmodel)
        self.add_columns( self.isolist)
        
        cdlistmodel = self.create_model_cdromdrives(self.settings["cdromdrives"])
        self.treeview_cdrives.set_model(cdlistmodel)
        self.add_columns_cdromdrives(self.treeview_cdrives, cdlistmodel)         
        
        cdmodel = self.combobox_cdromdrive.get_model()
        cdmodel.clear()

        cdmodel.append(row=(None,))
        for dev in self.settings["cdromdrives"]:
            name = dev[0]
            device = dev[1]
            cdmodel.append(row=(name,))
            self.combobox_cdromdrive.set_model(cdmodel)
            
        self.add_columns_statefiles(self.treeview_statefiles)
        self.add_columns_capture(self.treeview_capture)
        self.add_columns_screenshots(self.iconview_screenshots)
        
        try:
            self.filechooserbutton_imagecdrom.add_shortcut_folder(self.settings["imagespath"])
            #print "added imagespath to filechooserbutton_imagecdrom"
        except:
            #print "imagespath allready on filechooserbutton_imagecdrom"
            pass
        try:
            self.filechooserbutton_imagecdrom.add_shortcut_folder("/dev/")
        except:
            pass        
        self.filechooserbutton_imagecdrom.add_filter(filter_shall(self.settings))
        self.filechooserbutton_imagecdrom.add_filter(filter_devs(self.settings))
        
        self.filechooserbutton_imagecdrom.set_filter(filter_shall(self.settings))
        
        try:
            self.filechooserbutton_hda.add_shortcut_folder(self.settings["imagespath"])
        except:
            pass
        try:
            self.filechooserbutton_hda.add_shortcut_folder("/dev/")
        except:
            pass        
        self.filechooserbutton_hda.add_filter(filter_shall(self.settings))
        self.filechooserbutton_hda.add_filter(filter_devs(self.settings))
        
        self.filechooserbutton_hda.set_filter(filter_shall(self.settings))
        try:        
            self.filechooserbutton_hdb.add_shortcut_folder(self.settings["imagespath"])
        except:
            pass 
        try:        
            self.filechooserbutton_hdb.add_shortcut_folder("/dev/")
        except:
            pass             
        self.filechooserbutton_hdb.add_filter(filter_shall(self.settings))
        self.filechooserbutton_hdb.add_filter(filter_devs(self.settings))
        
        self.filechooserbutton_hdb.set_filter(filter_shall(self.settings))
        try:         
            self.filechooserbutton_hdc.add_shortcut_folder(self.settings["imagespath"])
        except:
            pass   
        try:              
            self.filechooserbutton_hdc.add_shortcut_folder("/dev/")
        except:
            pass               
        self.filechooserbutton_hdc.add_filter(filter_shall(self.settings))
        self.filechooserbutton_hdc.add_filter(filter_devs(self.settings))
        
        self.filechooserbutton_hdc.set_filter(filter_shall(self.settings))
        try:        
            self.filechooserbutton_hdd.add_shortcut_folder(self.settings["imagespath"])
        except:
            pass    
        try:             
            self.filechooserbutton_hdd.add_shortcut_folder("/dev/")
        except:
            pass         
        self.filechooserbutton_hdd.add_filter(filter_shall(self.settings))
        self.filechooserbutton_hdd.add_filter(filter_devs(self.settings))
        
        self.filechooserbutton_hdd.set_filter(filter_shall(self.settings))
        try:        
            self.filechooserbutton_fda.add_shortcut_folder(self.settings["imagespath"])
        except:
            pass   
        try:              
            self.filechooserbutton_fda.add_shortcut_folder("/dev/")
        except:
            pass         
        self.filechooserbutton_fda.add_filter(filter_devs(self.settings))
        self.filechooserbutton_fda.add_filter(filter_shall(self.settings))
        self.filechooserbutton_fda.set_filter(filter_shall(self.settings))
        try:        
            self.filechooserbutton_fdb.add_shortcut_folder(self.settings["imagespath"])
        except:
            pass   
        try:              
            self.filechooserbutton_fdb.add_shortcut_folder("/dev/")
        except:
            pass         
        self.filechooserbutton_fdb.add_filter(filter_shall(self.settings))
        self.filechooserbutton_fdb.add_filter(filter_devs(self.settings))
        self.filechooserbutton_fdb.set_filter(filter_shall(self.settings))

        self.radiobutton_network_usermode.set_active(True)
        self.toggle_networkuse(self.radiobutton_network_usermode)
        
        
        netmodel = self.create_model_networkcards([])        
        self.treeview_networkcards.set_model(netmodel)
        self.add_columns_networkcards(self.treeview_networkcards,netmodel)  
        
        redirmodel = self.create_model_redirects([])    
        self.treeview_redirects.set_model(redirmodel) 
        self.add_columns_redirects(self.treeview_redirects,redirmodel)             
        
        gtk.threads_leave()  
        self.get_soundhw()
        
        gtk.threads_enter()
        self.refresh_soundhw() 
        gtk.threads_leave()
        
        self.get_machine()
        
        gtk.threads_enter()
        self.refresh_machineslist()
        
        
        ## initializing the systems listbox
        sysmodel = self.combobox_emusystem.get_model()
        try:
            sysmodel.clear()
        except:
            #print "nothing to clear"
            pass
        for commands in self.settings["qemuapps"]:
            name = commands[0]
            path = commands[1]
            #print "path: " + name + " - " + path
            
            sysmodel.append(row=(name,))
        self.combobox_emusystem.set_model(sysmodel)  
        self.combobox_emusystem.set_active(0);      
        self.currimage_settings.emusystem = self.settings["qemuapps"][0][0]
        #gtk.threads_leave()
        
        self.init_joblist()
        
        #gtk.threads_enter()
        self.mainwindow.present()
        gtk.threads_leave()   
    
    def filter_nodirs(self, filter_info):
        file = filter_info[1]
        print file
        if path.isdir(file):
            print "isdir"
            return False
        else:
            print "nodir"
            return True    
    
    def create_model_ilist(self, data):
        store = gtk.ListStore(gobject.TYPE_STRING,
                          gtk.gdk.Pixbuf)
        config_icon = gtk.Image()
        try:        
            for item in data:
                iter = store.append()
                itempath = item[0]
                haveconfig = item[1]
                if haveconfig == True:
                    config_icon = gtk.gdk.pixbuf_new_from_file(self.pixmapdir + '/gtk-yes.png')
                else:
                    config_icon = gtk.gdk.pixbuf_new_from_file(self.pixmapdir + '/gtk-no.png')                
                store.set(iter, 0, itempath, 1, config_icon)
    
            return store
        except:
            print "recieved no correct data for treemodel creation"
               
        
    def add_columns(self, treeview):
        model = self.isolist.get_model()
        renderer = gtk.CellRendererToggle()
        column = gtk.TreeViewColumn(_('Qemu Images'), gtk.CellRendererText(), text=0)
        column.set_resizable(True)
        column2 = gtk.TreeViewColumn(_('configured'), gtk.CellRendererPixbuf(), pixbuf=1)
        column2.set_resizable(True)
        treeview.append_column(column)
        treeview.append_column(column2)
        
    def create_model_statefiles(self, data):
        store = gtk.ListStore(gobject.TYPE_STRING)
        data.sort()
        try:
            for item in data:
                iter = store.append()
                filename = item
                string.strip(filename)
                #print filename

                store.set(iter, 0, filename)
            return store          
        except:
            print "recieved no correct data for statefiles treemodel creation"
       
    def add_columns_statefiles(self, treeview):
    
        renderer = gtk.CellRendererToggle()
        column = gtk.TreeViewColumn(_('Name'), gtk.CellRendererText(), text=0)
        treeview.append_column(column)
        
    def refresh_statefiles(self, *widget):  
        try:
            model = self.treeview_statefiles.get_model()
            model.clear()
        except:
            pass          
        data = self.stateshandler.load_statefiles()
        #print "Statefiles data: " + str(data)
        ddata = self.stateshandler.statefiles_displaynames(data)
        model = self.create_model_statefiles(ddata)
        self.treeview_statefiles.set_model(model)  

    def delete_statefile_clicked(self, widget):
        error = False
        try:
            model, iter = self.treeview_statefiles.get_selection().get_selected()
            row = model[iter]
            item = row[0]            
        except:
            error = True
            dialogtext = "<b>No file selected to delete</b>"
            delrequest = Dialog("message", dialogtext, "Message")
        if error != True:
            realname = str(item.replace ( ' ', '_' ))
            dialogtext = "<b>Do you really want to delete the\n"\
            "selected state file\n"\
            "from your harddisk?</b>\n\n"\
            "File: " + realname
            dialogtitle = "Confirm delete"
            delrequest = Dialog("request", dialogtext, dialogtitle, self.delete_statefile, [realname])        

    def delete_statefile(self, filename):
        self.stateshandler.delete_statefile(filename)
        self.refresh_statefiles()        
                
    def load_statefile_clicked(self, widget):
        model, iter = self.treeview_statefiles.get_selection().get_selected()
        row = model[iter]
        item = row[0]
        realname = str(item.replace ( ' ', '_' ))
        self.statefile = realname
        self.run_qemu(widget)
        return self.statefile
       
    def create_model_capture(self, data):
        store = gtk.ListStore(gobject.TYPE_STRING)
        data.sort()
        try:
            for item in data:
                iter = store.append()
                filename = item
                string.strip(filename)
                store.set(iter, 0, filename)
            return store          
        except:
            print "recieved no correct data for captures treemodel creation"
       
    def add_columns_capture(self, treeview):
    
        renderer = gtk.CellRendererToggle()
        column = gtk.TreeViewColumn(_('Name'), gtk.CellRendererText(), text=0)
        treeview.append_column(column)
 
    def refresh_captures(self, *widget): 
        try:
            model = self.treeview_capture.get_model()
            model.clear()
        except:
            pass           
        data = self.capturehandler.load_captures()
        ddata = self.capturehandler.captures_displaynames(data)
        model = self.create_model_capture(ddata)
        self.treeview_capture.set_model(model)
        #print "refreshed captures"
        
    def delete_capture_clicked(self, widget):
        error = False
        try:
            model, iter = self.treeview_capture.get_selection().get_selected()
            row = model[iter]
            item = row[0]            
        except:
            error = True
            dialogtext = "<b>" + _("No file selected to delete") + "</b>"
            delrequest = Dialog("message", dialogtext, "Message")
        if error != True:
            realname = str(item.replace ( ' ', '_' )) + ".wav"
            dialogtext = "<b>" + _("Do you really want to delete the\n"\
            "selected wave file\n"\
            "from your harddisk?") + "</b>\n\n"\
            "File: " + realname
            dialogtitle = _("Confirm delete")
            delrequest = Dialog("request", dialogtext, dialogtitle, self.delete_capture, [realname])        
        
        
    def delete_capture(self, filename):
        self.capturehandler.delete_capture(filename)
        self.refresh_captures()             
            
    def run_audioplayer(self, widget, event=None):
        if event == None or event.type == gtk.gdk._2BUTTON_PRESS:
            model, iter = self.treeview_capture.get_selection().get_selected()
            row = model[iter]
            audiofile = row[0]    
            realname = str(audiofile.replace ( ' ', '_' )) + ".wav"           
            if audiofile != "":
                self.capturehandler.run_audioplayer(realname)
                
    def pop_up_screenshots(self, widget, event):    
        if event.button == 3:            
            try:  
                path, renderer = self.iconview_screenshots.get_cursor()
                model = self.iconview_screenshots.get_model()
                iter = model.get_iter(path)        
                self.selected_screenshot = model.get_value(iter, 0)
                self.menu_popup_screenshots.popup( None, None, None, 3, 0)
            except:
                print "no screenshot selected"
        else:
            pass
        
    def screenshot_clicked(self, widget, event=None):
        if event.button == 3: 
            isbookmark = self.search_bookmark(self.currentimage, "index")
            if isbookmark != None:
                print "is bookmark: " + str(isbookmark) 
                self.item_use_as_icon.set_sensitive(True)
            else:
                self.item_use_as_icon.set_sensitive(False)          
            self.pop_up_screenshots(widget, event)
        elif event == None or event.type == gtk.gdk._2BUTTON_PRESS:
            self.run_imageviewer(widget, event)
        
    def scr2icon(self, *widget):
        print "use as icon!"
        isbookmark = self.search_bookmark(self.currentimage, "index")
        if isbookmark != None:
            currbookmark = isbookmark
        self.translated_name = self.currentimage.replace ( '/', '_' )    
        thisjobdir = self.screenshot_dir + "/" + self.translated_name
        screenshot = thisjobdir + "/" + self.selected_screenshot
        self.bookmarks[currbookmark].icon = screenshot
        self.storetofile(self.bookmarks, "bookmarks")
        self.refresh_bookmarks()
                                
    def create_model_screenshots(self, data):
        model = gtk.ListStore(str, gtk.gdk.Pixbuf)
        data.sort()
        for item in data:
            filename = item
            string.strip(filename)
            filepath = self.screenshot_dir + "/" + self.translated_name + "/" + filename
            pixbuf = gtk.gdk.pixbuf_new_from_file(filepath)
            pwidth = pixbuf.get_width()
            pheight = pixbuf.get_height()
            pfact = pwidth / 64
            oheight = pheight / pfact
            icon = pixbuf.scale_simple(64, oheight, 3)
            model.append([filename, icon])
        return model          

    def add_columns_screenshots(self, iconview=None):
        iniconview = iconview
        iniconview.set_text_column(0)
        iniconview.set_pixbuf_column(1)
                
    def refresh_screenshots(self, *widget):
        try:
            model = self.iconview_screenshots.get_model()
            model.clear()
        except:
            pass             
        data = self.sceenshothandler.load_screenshots()
        model = self.create_model_screenshots(data)
        self.iconview_screenshots.set_model(model)    
            
    def delete_screenshot_clicked(self, widget):
        error = False
        try:
            path, renderer = self.iconview_screenshots.get_cursor()
            model = self.iconview_screenshots.get_model()
            iter = model.get_iter(path)            
            screenshot = model.get_value(iter, 0)
        except:
            error = True
            dialogtext = "<b>No file selected to delete</b>"
            delrequest = Dialog("message", dialogtext, "Message")
        if error != True and screenshot != "":
            dialogtext = "<b>Do you really want to delete the\n"\
            "selected screenshot file\n"\
            "from your harddisk?</b>\n\n"\
            "File: " + screenshot
            dialogtitle = "Confirm delete"
            delrequest = Dialog("request", dialogtext, dialogtitle, self.delete_screenshot, [screenshot])        

    def delete_screenshot(self, filename):
        self.sceenshothandler.delete_screenshot(filename)
        self.refresh_screenshots() 

    def run_imageviewer(self, widget, event=None):
        if event == None or event.type == gtk.gdk._2BUTTON_PRESS:
            path, renderer = self.iconview_screenshots.get_cursor()
            model = self.iconview_screenshots.get_model()
            iter = model.get_iter(path)            
            screenshot = model.get_value(iter, 0)        
            if screenshot != "":
                self.sceenshothandler.run_imageviewer(screenshot)
                                
    ## actions when an image file was selected
    def treeview_selection(self, widget):
        model, iter = widget.get_selection().get_selected()
        row = model[iter]
        item = row[0]
        #print "item: %s" %item
        self.set_currentimage(item) 
        #print "item: %s" %item

    ## Function to set the current emu-image when clicking an Entry
    def set_currentimage(self, imgstring):
        if imgstring != None and imgstring != "":
            self.activate_launchbuttons(True)
            #print "Image: " + imgstring + " - selected"
            #_hid = self.entry_current_image.get_data('imageselection')
            _hid = self.imageselection
            #print "_hid: " + str(_hid)
            self.entry_current_image.handler_block(_hid)
    
            self.entry_current_image.set_text(imgstring)
            
            self.entry_current_image.handler_unblock(_hid)
            c = 0
            while self.entry_current_image.get_text() == "":
                c = c+1
                time.sleep(0.2)
                #print "waiting for selection"
                if c == 30: break 
            time.sleep(0.2)           
            self.select_image_from_path(imgstring)
        else:
            print "No image selected set current"
        return
            
            
    ## Universal Function to pickle-store the configfiles
    def storetofile(self, indata, filename):
        #print "storefile: " + filename
        dumpf = open(self.configdir + "/" + filename ,"w")        
        pickle.dump(indata, dumpf)
        dumpf.close()
        
    ## Universal Function to open the pickle'd configfiles        
    def getfromfile(self, filename):
        try:
            abs_path = path.abspath(self.configdir + "/" + filename)
            #print "opened file absolute path: " + abs_path 
            dumpf = open( abs_path ,"r")       
            outdata = pickle.load(dumpf)
            dumpf.close()
            return outdata        
        except:
            outdata  = { }
            print "File not found: " + str(abs_path)
            return outdata
       
    def search(self, model, iter, func, data):
       while iter:
           if func(model, iter, data):
               return iter
           result = search(model, model.iter_children(iter), func, data)
           if result: return result
           iter = model.iter_next(iter)
       return None
   
    ## Searches for the index for a 1 dimensional treemodel
    def search_combolist(self, combobox, searchstring):
        ct = 0
        outct = None
        cdmodel = combobox.get_model()
        for row in cdmodel:

            if searchstring == row[0]:
                outct = ct
            ct = ct + 1           
        return outct
        
    def fetch_image_info(self, image):
        info = _('no valid image selected')  
       
        try:
            command = "qemu-img info " + image
            info = os.popen(command).read()
        except:
            info = _('no valid image selected')  
        return info  
    
    ## Fetch the stored configuration for the selected image
    def restore_emu_config(self, widget=None):
        #print "Config called from widget: " + str(widget)
        self.currimage_settings = QemuConfig()
        self.image_settings = getfromfile("image_settings")
        haveconfig = False
        try:
            if self.image_settings[self.currentimage]:
                #print "Config found for :" + str(self.currentimage)
                haveconfig = True
              
            else:
                print "No config found for :" + str(self.currentimage)
                self.label_imgsettings.set_markup(_('<b>Create new Settings for Current selected image</b>'))
                haveconfig = False   
        except:
            self.label_imgsettings.set_markup(_('<b>Create new Settings for Current selected image</b>')) 
            print "No config found for :" + str(self.currentimage)
            haveconfig = False
        if haveconfig == True:
            if self.settings_autohide == True:
                #gtk.threads_enter()
                self.switch_settingsview("hide")
                #gtk.threads_leave()
            
            newvals = vars(self.image_settings[self.currentimage])
            oldvals = vars(self.currimage_settings)
            for key in newvals.keys():
                var = key
                val = newvals[key]
                oldvals[var] = val
            #print "loaded emu_config from file"
            #if widget != None:
            #    gtk.threads_enter()
            self.restoreaction = True
            refreshsettings = self.refresh_settings_gui()
            #print "refreshsettings: " + str(refreshsettings)
            self.restoreaction = False
            #while gtk.gdk.events_pending:
            #    print "waiting for gui refresh..."
            #    time.sleep(0.3)
            #while self.refresh_settings_gui():
            #    print "waiting for gui refresh..."
            #    time.sleep(0.3)
            #if widget != None:
            #    gtk.threads_leave()
                           
            self.toggle_storebuttons(False, "restore_emu_config")                
        else:
            if self.settings_autoshow == True:
                #gtk.threads_enter()
                self.switch_settingsview("show")
                #gtk.threads_leave()
            #time.sleep(0.2)
            self.toggle_storebuttons(True, "restore_emu_config") 
                    
    def refresh_settings_gui(self):
        self.stateshandler = qml_filehandlers.Statefiles(self.currentimage, self.settings)
        self.sceenshothandler = qml_filehandlers.Screenshots(self.currentimage, self.settings)
        self.capturehandler = qml_filehandlers.Wavecaptures(self.currentimage, self.settings)  
        self.translated_name = self.currentimage.replace ( '/', '_' )
        
        #gtk.threads_enter()
        
        self.refresh_statefiles()
        self.refresh_screenshots()
        self.refresh_captures()
        #gtk.threads_leave()
        
        self.statefile = None
        #self.restoreaction = True
        
        #gtk.threads_enter()
        self.label_imgsettings.set_markup(_('<b>Stored Settings for Current selected image</b>'))
         
        if self.currimage_settings.emusystem == "":    
            self.currimage_settings.emusystem = str(self.combobox_emusystem.get_active())           
               
        self.qemubin = self.emusystem_path_by_name(self.currimage_settings.emusystem)
        find_emusystem = self.search_combolist(self.combobox_emusystem, self.currimage_settings.emusystem)
        if find_emusystem != None:
            self.combobox_emusystem.set_active(find_emusystem)    
        else:
            self.combobox_emusystem.set_active(0)
            self.select_emusystem(self.combobox_emusystem)
        #gtk.threads_leave()
            
        ## Select the emu machine and setup GUI      
        self.machinecommand = self.machine_path_by_name(self.currimage_settings.machine) 
        #print "machine loaded: " + self.machinecommand
        
        #gtk.threads_enter()
        find_machine = self.search_combolist(self.combobox_machinetype, self.currimage_settings.machine)
        if find_machine != None:
            self.combobox_machinetype.set_active(find_machine)   
        else:
            self.get_machine()
            self.combobox_machinetype.set_active(0)
            self.select_machine(self.combobox_machinetype)
        if self.currimage_settings.kqemu_param ==  "-no-kqemu":
            self.radiobutton_nokqemu.set_active(True )
        elif self.currimage_settings.kqemu_param ==  "-kernel-kqemu":
            self.radiobutton_kkqemu.set_active(True )  
        else:
            self.radiobutton_kqemu.set_active(True )
        
        ## Select the bootmedium and setup GUI

        if self.currimage_settings.bootmedium ==  "cdrom":
            self.combobox_imagedevice.set_active(1);
        elif self.currimage_settings.bootmedium ==  "fda":
            self.combobox_imagedevice.set_active(2);              
        else:
            self.combobox_imagedevice.set_active(0);
            
        ## Select the cdrmedium and setup GUI    

        find_cdrmedium = self.search_combolist(self.combobox_cdromdrive, self.currimage_settings.cdrmedium)
        if find_cdrmedium != None:
            self.combobox_cdromdrive.set_active(find_cdrmedium)
        else:
            self.combobox_cdromdrive.set_active(0)   
            
        ## select cdrmode and setup GUI
          
        if self.currimage_settings.cdrmode == "mount":
            self.radiobutton_cdromtype1.set_active(True)
        elif self.currimage_settings.cdrmode == "image":
            self.radiobutton_cdromtype2.set_active(True)
        else:
            if self.radiobutton_cdromtype.get_active() == True:
                self.no_xtracdrom(self.radiobutton_cdromtype)
            else:
                self.radiobutton_cdromtype.set_active(True)
                                       
 
        ## Select the cdrimage and setup GUI 
        #print "_________ PAUSE __________________"
        #time.sleep(0.1)
        
        _hid = self.sighandlers['filechooserbutton_imagecdrom']
        self.filechooserbutton_imagecdrom.handler_block(_hid)        
        if self.currimage_settings.cdrimage != None and path.isfile(self.currimage_settings.cdrimage):
            self.filechooserbutton_imagecdrom.set_filename(str(self.currimage_settings.cdrimage))
        else:
            self.filechooserbutton_imagecdrom.set_current_folder(self.settings["imagespath"])                 
        self.filechooserbutton_imagecdrom.handler_unblock(_hid)

            
        ## Select the hda_image and setup GUI

        if self.currimage_settings.hda_image != None and path.isfile(self.currimage_settings.hda_image): 
            #gtk.threads_enter()
            self.filechooserbutton_hda.set_filename(str(self.currimage_settings.hda_image))    
            #gtk.threads_leave()
        else:
            #gtk.threads_enter()
            self.filechooserbutton_hda.set_current_folder(self.settings["imagespath"])          
            #gtk.threads_leave()
        ## Select the hdb_image and setup GUI

        if self.currimage_settings.hdb_image != None and path.isfile(self.currimage_settings.hdb_image):
            self.filechooserbutton_hdb.set_filename(str(self.currimage_settings.hdb_image))
        else:
            self.filechooserbutton_hdb.set_current_folder(self.settings["imagespath"])
              
        ## Select the hdc_image and setup GUI

        if self.currimage_settings.hdc_image != None and path.isfile(self.currimage_settings.hdc_image):
            self.filechooserbutton_hdc.set_filename(str(self.currimage_settings.hdc_image))
        else:
            self.filechooserbutton_hdc.set_current_folder(self.settings["imagespath"])   
      
        ## Select the hdd_image and setup GUI

        if self.currimage_settings.hdd_image != None and path.isfile(self.currimage_settings.hdd_image):
            self.filechooserbutton_hdd.set_filename(str(self.currimage_settings.hdd_image))
        else:
            self.filechooserbutton_hdd.set_current_folder(self.settings["imagespath"])    
        
        ## Select the fda_image and setup GUI       

        if self.currimage_settings.fda_image != None and path.isfile(self.currimage_settings.fda_image):
            self.filechooserbutton_fda.set_filename(str(self.currimage_settings.fda_image))
        else:
            self.filechooserbutton_fda.set_current_folder(self.settings["imagespath"])

        ## Select the fdb_image and setup GUI

        if self.currimage_settings.fdb_image != None and path.isfile(self.currimage_settings.fdb_image):
            self.filechooserbutton_fdb.set_filename(str(self.currimage_settings.hdb_image))
        else:
            self.filechooserbutton_fdb.set_current_folder(self.settings["imagespath"])
        
        ## Select the soundhw and setup GUI    

        find_soundhw = self.search_combolist(self.combobox_audiohw, self.currimage_settings.soundhw)
        #print "find_soundhw " + str(find_soundhw)
        if find_soundhw:
            self.combobox_audiohw.set_active(find_soundhw)     
        else:
            self.combobox_audiohw.set_active(0)
        
        ## Select the vgamode and setup GUI    

        if self.currimage_settings.vgamode == "std-vga":
            self.checkbutton_usevga.set_active(True)
        else:
            self.checkbutton_usevga.set_active(False)  

        ## Select usevnc and setup GUI

        self.checkbutton_startvnc.set_active(self.currimage_settings.usevnc)
        
                        
        ## Select vncdisplay and setup GUI
         
        self.spinbutton_vncdisplay.set_value(self.currimage_settings.vncdisplay)

        ## Select useusb and setup GUI
     
        self.checkbutton_enable_usb.set_active(self.currimage_settings.useusb)
        
        ## Select novga and setup GUI

        self.checkbutton_novga.set_active(self.currimage_settings.novga)
        
        ## Select fullscreen mode and setup GUI

        self.checkbutton_fullscreen.set_active(self.currimage_settings.fullscreenmode)
        
        ## Select dlinuxboot and setup GUI

        self.checkbutton_dlinuxboot.set_active(self.currimage_settings.dlinuxboot)
        self.frame_dlinuxboot.hide()
        self.button_hide_dlinuxboot.set_label(_("Show settings"))
        
        ## Select the kernelimage and setup GUI

        if self.currimage_settings.kernelimage != None and path.isfile(self.currimage_settings.kernelimage): 
            self.filechooserbutton_kernelimage.set_filename(str(self.currimage_settings.kernelimage))    

        ## Select the initrd and setup GUI

        if self.currimage_settings.initrd != None and path.isfile(self.currimage_settings.initrd): 
            self.filechooserbutton_initrd.set_filename(str(self.currimage_settings.initrd))    

        ## Select extra kernel parameters and setup GUI
            
        self.entry_kernelcmd.set_text(self.currimage_settings.kernelcmd)                                                                         
                                      
                  
        ## Select snapshotmode and setup GUI

        self.checkbutton_snapshotmode.set_active(self.currimage_settings.snapshotmode)
        
        ## Select localtimemode and setup GUI

        self.checkbutton_localtime.set_active(self.currimage_settings.localtimemode)

        ## Select memory and setup GUI
           
        self.spinbutton_memory.set_value(self.currimage_settings.memory)
        #print "memory loaded: " + str(self.currimage_settings.memory)
        
        ## Select customoptions and setup GUI
          
        self.entry_additional_options.set_text(self.currimage_settings.customoptions)                                                                         
         
        ## Select bootfromcd and setup GUI
        
        ## Select networkmode and setup GUI
            
        if self.currimage_settings.networkmode == "nonet":
            self.radiobutton_network_nonet.set_active(True)
        elif self.currimage_settings.networkmode == "usermode" or self.currimage_settings.networkmode == "":
            self.radiobutton_network_usermode.set_active(True)
            self.usernetwork = self.currimage_settings.usernetwork
        elif self.currimage_settings.networkmode == "custom":
            self.radiobutton_network_customsetup.set_active(True)
        else:
            self.radiobutton_network_usermode.set_active(True)
                                                                                     

        ## Select hostname and setup GUI
        #print "Network hostname: " + str(self.currimage_settings.usernetwork.hostname)
        #self.entry_network_hostname.set_text(self.currimage_settings.network_hostname)
        self.entry_network_hostname.set_text(str(self.currimage_settings.usernetwork.hostname))            
                
        ## Select usesmb and setup GUI
           
        self.checkbutton_smbshare.set_active(self.currimage_settings.smbsettings.usesmb)
        self.filechooserbutton_network_smbshare.unselect_all()
        
        ## Select smbpath and setup GUI
         
        if os.path.isfile(str(self.currimage_settings.smbsettings.smbpath)):
            self.filechooserbutton_network_smbshare.set_filename(str(self.currimage_settings.smbsettings.smbpath))
            #self.network_hostname = "emu"
        
        ## Select vlan and setup GUI
        self.spinbutton_usernet_vlan.set_value(int(self.currimage_settings.usernetwork.vlan))

        #print "networksettings loaded"
        #print "networkredirs loaded"
        self.refresh_networkcards()
        self.refresh_redirects() 
        #print "config restored for this image"
        #self.toggle_storebuttons(False, "refresh_settings_gui")
        #print "gui updated for this image"   
        #gtk.threads_leave()
        
        self.restoreaction = False
        return True
            
    ## Store changed settings for current image to file        
    def store_emu_settings(self, widget):
        if self.currentimage != "":
            if self.currimage_settings.kqemu_param == "":
                store_kqemu_param = "NULL"
            else:
                store_kqemu_param = self.currimage_settings.kqemu_param
            if self.currimage_settings.emusystem == "":    
                self.currimage_settings.emusystem = str(self.combobox_emusystem.get_active())
                           
            self.image_settings[self.currentimage] = self.currimage_settings
            storedata = self.image_settings
            filename = "image_settings"
            self.storetofile(storedata, filename)
            self.refresh_ilist()
            self.refresh_bookmarks()
            self.label_imgsettings.set_markup(_('<b>Stored Settings for Current selected image</b>'))
        else:
            print "no image selected"
        self.toggle_storebuttons(False, "store_emu_settings")

    def delete_event(self, widget, event, data=None):
        
        if tray != None and self.settings["closetotray"] == True:
            print "close to tray enabled, only hiding window"
            self.position = widget.get_position()
            widget.hide()
            return True
        else:  
            print "close to tray not enabled, exiting.."  
            self.doexit = True
            self.dorun = False
            time.sleep(0.5)
            print "exit"
            exit
            gtk.main_quit()
            return False
    
    def exit_app_event(self, widget, *event):
        self.doexit = True
        self.dorun = False
        time.sleep(0.5)        
        print "exit"
        exit
        gtk.main_quit()  
        return False    
    
    ## performs all actions when selecting an image
    def select_bootimage(self, widget):
        active = widget.get_active()
        model = widget.get_model()
        row = model[active]
        item = row[0]
        self.currimage_settings.bootmedium = item
        if self.currimage_settings.bootmedium != "hda":
            self.filechooserbutton_hda.set_sensitive( True )

        if self.currimage_settings.bootmedium != "fda":       
            self.filechooserbutton_fda.set_sensitive( True )
        if self.currimage_settings.bootmedium == "hda":
            self.currimage_settings.boot_driveletter = "c"
            self.activate_xtracdrom(True)      
            self.filechooserbutton_hda.unselect_all()
            self.filechooserbutton_hda.set_sensitive( False )   
        elif self.currimage_settings.bootmedium == "cdrom":            
            self.currimage_settings.boot_driveletter = "d"
            self.activate_xtracdrom(False) 
        elif self.currimage_settings.bootmedium == "fda":            
            self.currimage_settings.boot_driveletter = "a"
            self.activate_xtracdrom(True)  
            self.filechooserbutton_fda.unselect_all() 
            self.filechooserbutton_fda.set_sensitive( False )             
        else:
            self.currimage_settings.boot_driveletter = "c"
            self.activate_xtracdrom(True) 
        #print "-" + self.currimage_settings.bootmedium + " -boot " + self.currimage_settings.boot_driveletter
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "select_bootimage")

    ## perform gui actions when using of extra cdrom device is not possible
    ## in current configuration
    def no_xtracdrom(self, widget):
        active = widget.get_active()
        #print "no xtracd"
        if active == True:
            #self.combobox_cdromdrive.set_sensitive( False );
            self.combobox_cdromdrive.hide()
            #self.filechooserbutton_imagecdrom.set_sensitive(False)
            self.filechooserbutton_imagecdrom.hide()
            #self.button_eject_imagecdrom.set_sensitive(False)
            self.button_eject_imagecdrom.hide()
            self.currimage_settings.cdrimage = None
            self.currimage_settings.cdrmedium = None
            self.currimage_settings.cdrmode = "nocd"
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "no_xtracdrom")
        
    ## Set Variables and perform gui actions when changing "combobox_cdromdrive"
    def select_mountcdrom(self, widget):
        active = widget.get_active()
        if active == True:
            #print "CDR-Drive %s" %active
            #self.combobox_cdromdrive.set_sensitive( True )
            self.combobox_cdromdrive.show()
            #self.filechooserbutton_imagecdrom.set_sensitive(False)
            self.filechooserbutton_imagecdrom.hide()
            #self.button_eject_imagecdrom.set_sensitive(False)
            self.button_eject_imagecdrom.hide()
            self.currimage_settings.cdrimage = None
            self.currimage_settings.cdrmode = "mount"
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "select_mountcdrom")       
        
    def select_imagecdrom(self, widget):
        active = widget.get_active()   
        if active == True:  
            #self.combobox_cdromdrive.set_sensitive(False)
            self.combobox_cdromdrive.hide()
            #self.filechooserbutton_imagecdrom.set_sensitive(True)
            self.filechooserbutton_imagecdrom.show()
            #self.button_eject_imagecdrom.set_sensitive(True)
            self.button_eject_imagecdrom.show()
            self.currimage_settings.cdrmedium = None
            self.currimage_settings.cdrmode = "image"
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "select_imagecdrom")
        
    def select_cdromdrive1(self, widget):
        active = widget.get_active()
        model = widget.get_model()
        row = model[active]
        item = row[0]
        self.currimage_settings.cdrmedium = item
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "select_cdromdrive1")
    
    def select_cdromdrive(self, widget):  
        active = widget.get_active()
        model = widget.get_model()
        row = model[active]
        item = row[0]          
        self.currimage_settings.cdrmedium = item
        try:
            actct_cdrmedium = self.search_combolist(self.combobox_cdromdrive, self.currimage_settings.cdrmedium)
            self.combobox_cdromdrive.set_active(actct_cdrmedium)    
        except:
            pass
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "select_cdromdrive")
                    
    def load_cdromdrive(self, widget):   
        try:
            active = widget.get_active()
            model = widget.get_model()
            row = model[active]
            item = row[0]
        except:
            item = widget.get_filename()        
        self.filechooserbutton_cdromdrive.unselect_all()
        self.currimage_settings.cdrmedium = item
        if self.currimage_settings.cdrmedium != "None" and self.currimage_settings.cdrmedium != None: 
            self.filechooserbutton_cdromdrive.set_filename(str(self.currimage_settings.cdrmedium))
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "load_cdromdrive")
        
    def select_cdromimage1(self, widget):
        active = widget.get_active()
        model = widget.get_model()
        row = model[active]
        item = row[0]
        self.currimage_settings.cdrimage = item
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "select_cdromimage1")
        
    def load_cdromimage(self, widget, event=None): 
        #if event != None:
        #    print "Event: " + str(event)
        item = str(widget.get_filename())
        self.currimage_settings.cdrimage = item
        tooltips.set_tip(self.eventbox_imagecdrom, str(self.currimage_settings.cdrimage))
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "load_cdromimage")

    def load_xtra_image(self, widget):
        wname = widget.get_name()
        item = str(widget.get_filename())
        if wname == 'filechooserbutton_hda':
            self.currimage_settings.hda_image = item
            tooltips.set_tip(self.eventbox_filechooserbutton_hda, str(self.currimage_settings.hda_image))
        if wname == 'filechooserbutton_hdb':
            self.currimage_settings.hdb_image = item 
            tooltips.set_tip(self.eventbox_filechooserbutton_hdb, str(self.currimage_settings.hdb_image))
            
        if wname == 'filechooserbutton_hdc':
            self.currimage_settings.hdc_image = item  
            tooltips.set_tip(self.eventbox_filechooserbutton_hdc, str(self.currimage_settings.hdc_image))
           
        if wname == 'filechooserbutton_hdd':
            self.currimage_settings.hdd_image = item 
            tooltips.set_tip(self.eventbox_filechooserbutton_hdd, str(self.currimage_settings.hdd_image))

        if wname == 'filechooserbutton_fda':
            self.currimage_settings.fda_image = item  
            tooltips.set_tip(self.eventbox_filechooserbutton_fda, str(self.currimage_settings.fda_image))
                
        if wname == 'filechooserbutton_fdb':
            self.currimage_settings.fdb_image = item    
            tooltips.set_tip(self.eventbox_filechooserbutton_fdb, str(self.currimage_settings.fdb_image))                 
              
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "load_xtra_image")
        return True
            
    def select_emusystem(self, widget):
        active = widget.get_active()
        model = widget.get_model()
        row = model[active]
        item = row[0]
        self.currimage_settings.emusystem = item 
        self.qemubin = self.emusystem_path_by_name(self.currimage_settings.emusystem)
        self.get_soundhw()
        self.refresh_soundhw() 
        
        self.get_machine()
        self.refresh_machineslist()

        if self.restoreaction == False:
            self.toggle_storebuttons(True, "select_emusystem")       
        
    def select_machine(self, widget):
        active = widget.get_active()
        model = widget.get_model()
        row = model[active]
        item = row[0]
        self.currimage_settings.machine = item
        self.machinecommand = self.machine_path_by_name(self.currimage_settings.machine) 
        self.get_soundhw()
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "select_machine")      

    def select_soundhw(self, widget):
        active = widget.get_active()
        model = widget.get_model()
        row = model[active]
        item = row[0]
        self.currimage_settings.soundhw = item   
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "select_soundhw")

    def set_vgamode(self, *widget):
        if self.checkbutton_usevga.get_active() == True:
                self.currimage_settings.vgamode = "std-vga"
        else:
                self.vgamode = "default"
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "set_vgamode")
        
    def set_novga(self, *widget):
        self.currimage_settings.novga = self.checkbutton_novga.get_active()
        #print "No VGA state changed: " + str(self.currimage_settings.novga)
        if self.currimage_settings.novga == False:
            self.checkbutton_usevga.set_sensitive(True)
            self.checkbutton_startvnc.set_sensitive(True)
            self.spinbutton_vncdisplay.set_sensitive(True)
            self.checkbutton_fullscreen.set_sensitive(True)
        else:
            self.checkbutton_usevga.set_active(False)
            self.checkbutton_usevga.set_sensitive(False)
            self.checkbutton_startvnc.set_active(False)
            self.checkbutton_startvnc.set_sensitive(False)
            self.spinbutton_vncdisplay.set_sensitive(False)
            self.checkbutton_fullscreen.set_active(False)
            self.checkbutton_fullscreen.set_sensitive(False)
            
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "set_novga")  
        
    def set_fullscreen(self, *widget):
        fullscreenmode = self.checkbutton_fullscreen.get_active()
        self.currimage_settings.fullscreenmode = fullscreenmode
    
             
                
    def set_startvnc(self, *widget):
        if self.checkbutton_startvnc.get_active() == True:
                self.currimage_settings.usevnc = True
        else:
                self.currimage_settings.usevnc = False
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "set_startvnc")   
                
    def set_vncdisplay(self, *widget):
        value = self.spinbutton_vncdisplay.get_text()
        c = 0
        while value == "" and c < 100:
            c = c + 1
            return False
        valint = int(value)
        self.currimage_settings.vncdisplay = valint
        #print valint
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "set_vncdisplay")
        
    def set_memory(self, widget, *data):
        value = self.spinbutton_memory.get_text()
        c = 0
        while value == "" and c < 100:
            c = c + 1
            return False
        valint = int(value)
        self.currimage_settings.memory = valint
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "set_memory")
        
    def set_usb(self, *widget):
        self.currimage_settings.useusb = self.checkbutton_enable_usb.get_active()
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "set_usb")
        
    def set_dlinuxboot(self, *widget):
        self.currimage_settings.dlinuxboot = self.checkbutton_dlinuxboot.get_active()
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "set_dlinuxboot")
        
    def load_kernelimage(self, widget): 
        item = widget.get_filename() 
        self.currimage_settings.kernelimage = item
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "load_kernelimage")
        
    def load_initrd(self, widget): 
        item = widget.get_filename() 
        self.currimage_settings.initrd = item
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "load_initrd")
        
    def set_kernelcmd(self, widget): 
        item = widget.get_text() 
        self.currimage_settings.kernelcmd = item
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "set_kernelcmd")             
        
    def set_snapshotmode(self, *widget):
        self.currimage_settings.snapshotmode = self.checkbutton_snapshotmode.get_active()
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "set_snapshotmode")   
        
    def set_localtimemode(self, *widget):
        self.currimage_settings.localtimemode = self.checkbutton_localtime.get_active()
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "set_localtimemode")     
     
    def set_customoptions(self, widget):
        self.currimage_settings.customoptions = widget.get_text()
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "set_customoptions")   
                        
    def cdrom_path_by_name(self, name):
        cdrdrives = self.settings["cdromdrives"]
        path = "None"
        dev = "None"
        for setting in cdrdrives:
            if setting[0] == name:
                path = setting[1]
                #print "cdrom path for " + str(name) + ": " + str(path)
        return path
        
    def soundhw_path_by_name(self, name):
        audiohws = self.audio_devlist
        path = "None"
        for setting in audiohws:
            if setting[1] == name:
                dev = setting[0]
                #print "sounddev: " + dev
        return dev        
        
    def cdrom_name_by_path(self, path):
        cdromdrives = self.settings["cdromdrives"]
        path = "None"
        for setting in cdromdrives:
            if setting[1] == path:
                name = setting[0]
                #print "name: " + name
                #print "cdrom name for " + str(path) + ": " + str(name)
        return path
        
    def emusystem_path_by_name(self, name):
        qemuapps = self.settings["qemuapps"]
        path = "None"
        for setting in qemuapps:
            if setting[0] == name:
                path = setting[1]
                #print "path: " + path
        return path        
        
    def machine_path_by_name(self, name):
        machines = self.machineslist
        path = ""
        for setting in machines:
            #print "found: " + str(setting[0])
            if setting[1] == name:
                path = setting[0]
                #print "machinecommand: " + str(path)
        return path         

    def set_nokqemu(self, widget):
        self.currimage_settings.kqemu_param = "-no-kqemu"
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "set_nokqemu")
        
    def set_kqemu(self, widget):
        self.currimage_settings.kqemu_param = ""
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "set_kqemu")
        
    def set_kkqemu(self, widget):
        self.currimage_settings.kqemu_param = "-kernel-kqemu"
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "set_kkqemu")
        
    def delete_openimagedialog(self):
        pass
        
    def fileChooser_emuimage(self, widget):
        
        dic = {"on_filechooserdialog_openimage_destroy" : self.delete_openimagedialog
        , "on_filechooserdialog_openimage_response" : self.fileChooser_emuimage_response
        }
        
        self.filechooserdialog_openimage = gtk.glade.XML(self.gladefile, "filechooserdialog_openimage", APP)
        self.filechooserdialog_openimage.signal_autoconnect(dic)
        
    def fileChooser_response(self, widget, response, *entry):
        #print 'widget: ' + str(widget)
        #print 'response: ' + str(response)
        file = None
        entrytext = widget.get_filename()
        #print 'entry: ' + entrytext
        if response == gtk.RESPONSE_OK:
            file = widget.get_filename ()
            widget.destroy ()
            #print 'File selected: ' + file       
            #self.set_currentimage(file)
            return file      
        elif response == gtk.RESPONSE_CANCEL:
            widget.destroy ()   
            #return None               
        return file        

    def fileChooser_emuimage_response(self, widget, response, *entry):
        #print 'widget: ' + str(widget)
        #print 'response: ' + str(response)
        entrytext = widget.get_filename()
        #print 'entry: ' + entrytext
        if response == gtk.RESPONSE_OK:
            file = widget.get_filename ()
            widget.destroy ()
            #print 'File selected: ' + file       
            self.set_currentimage(file)
            return file      
        elif response == gtk.RESPONSE_CANCEL:
            widget.destroy ()   
            return None               
        return None    

    def activate_launchbuttons(self, switchstate):
        self.toolbutton_launch.set_sensitive(switchstate)
        self.combobox_runmode.set_sensitive(switchstate)
            
    def activate_xtracdrom(self, switchstate):
        self.combobox_cdromdrive.set_sensitive(switchstate)  
        self.radiobutton_cdromtype1.set_sensitive(switchstate)
        self.radiobutton_cdromtype2.set_sensitive(switchstate)    
        if switchstate == True:
            self.radiobutton_cdromtype.set_active( False )
        elif switchstate == False:
            self.radiobutton_cdromtype.set_active( True )             
    
 ## Function to prebuild the qemu network commands
    def qemu_network_command(self):    
        qemu_netcommand = " "
        if self.currimage_settings.networkmode == "nonet":
            qemu_netcommand = " -net none"
        elif self.currimage_settings.networkmode == "usermode" or self.currimage_settings.networkmode == "":
            qemu_netcommand = self.currimage_settings.usernetwork.create_qemucommand()
            #qemu_netcommand = " -net nic,vlan=0 -net user,vlan=0,hostname=" + self.network_hostname            
        elif self.currimage_settings.networkmode == "custom":
            for netcard in self.currimage_settings.networksetup:
                command = netcard.create_qemucommand()
                qemu_netcommand = qemu_netcommand + command
            for redir in self.currimage_settings.networkredirs:
                command = redir.create_qemucommand()
                qemu_netcommand = qemu_netcommand + command
                
        if self.currimage_settings.networkmode != "nonet":
                command = self.currimage_settings.smbsettings.create_qemucommand()
                qemu_netcommand = qemu_netcommand + command

        return qemu_netcommand
    
 ## Function to build the qemu command
    def qemu_runmode(self, *widget):    
        actmode = str(self.combobox_runmode.get_active())
        model = self.combobox_runmode.get_model()
        row = model[actmode]
        runmode  = row[0]    
        return runmode
    
 ## Function to build the qemu command
    def qemu_command(self, rmode):
    
        machinecommand = ""
        memorycommand = " -m 128"
        cdrcommand = ""
        soundhwcommand = ""
        novgacommand = ""
        vgacommand = ""
        fullscreencommand = ""
        vnc_command = "" 
        usb_command = ""
        snapsot_command = ""
        localtime_command = ""
        hda_command = ""
        hdb_command = ""
        hdc_command = ""
        hdd_command = ""
        fda_command = ""
        fdb_command = ""
        networkoptions = ""
        customoptions = ""
        bootdriveletter = "c"
        netcommand = ""
        basecommand = ""
        statecommand = ""
        if self.machinecommand != "":
            machinecommand = " -M " + self.machinecommand
        
        memorycommand = " -m " + str(self.currimage_settings.memory)
        
        #print "mode: " + self.currimage_settings.cdrmode        
        if self.currimage_settings.cdrmode == "mount" and self.currimage_settings.cdrmedium != None and self.currimage_settings.cdrmedium != "None" and self.currimage_settings.cdrmedium != "":
            cdrpath = self.cdrom_path_by_name(self.currimage_settings.cdrmedium)
            cdrcommand = " -cdrom " + cdrpath
        elif self.currimage_settings.cdrmode == "image" and self.currimage_settings.cdrimage != None and self.currimage_settings.cdrimage != "None" and self.currimage_settings.cdrimage != "":
            cdrcommand = " -cdrom " + self.currimage_settings.cdrimage            

        if self.currimage_settings.soundhw != "" and self.currimage_settings.soundhw != "None":
            soundhw = self.soundhw_path_by_name(self.currimage_settings.soundhw)
            soundhwcommand = " -soundhw " + soundhw

        if self.currimage_settings.novga == True:
            novgacommand = " -nographic" 
            
        if self.currimage_settings.fullscreenmode == True: 
            fullscreencommand = " -full-screen"         
            
        if self.currimage_settings.vgamode == "std-vga":
            vgacommand = " -std-vga"

        if self.currimage_settings.usevnc == True:
            vnc_command = " -usbdevice tablet -vnc " + str(self.currimage_settings.vncdisplay)              
             
        if self.currimage_settings.useusb == True:
            usb_command = " -usb" 

        if self.currimage_settings.snapshotmode == True:
            snapsot_command = " -snapshot"  

        if self.currimage_settings.localtimemode == True:
            localtime_command = " -localtime"                             
        
        if self.currimage_settings.hda_image != None and self.currimage_settings.hda_image != "" and self.currimage_settings.hda_image != "None":
            hda_command = " -hda " + str(self.currimage_settings.hda_image)
        
        if self.currimage_settings.hdb_image != None and self.currimage_settings.hdb_image != "" and self.currimage_settings.hdb_image != "None":
            hdb_command = " -hdb " + str(self.currimage_settings.hdb_image)
        
        if self.currimage_settings.hdc_image != None and self.currimage_settings.hdc_image != "" and self.currimage_settings.hdc_image != "None":
            hdc_command = " -hdc " + str(self.currimage_settings.hdc_image)          
        
        if self.currimage_settings.hdd_image != None and self.currimage_settings.hdd_image != "" and self.currimage_settings.hdd_image != "None":
            hdd_command = " -hdd " + str(self.currimage_settings.hdd_image)
        
        if self.currimage_settings.fda_image != None and self.currimage_settings.fda_image != "" and self.currimage_settings.fda_image != "None":
            fda_command = " -fda " + str(self.currimage_settings.fda_image)
                    
        if self.currimage_settings.fdb_image != None and self.currimage_settings.fdb_image != "" and self.currimage_settings.fdb_image != "None":
            fdb_command = " -fdb " + str(self.currimage_settings.fdb_image)

        if self.statefile != None:
            thisjobdir = self.statefiles_dir + "/" + self.translated_name
            statefile_path = path.abspath(self.statefiles_dir + "/" + self.translated_name + "/" + self.statefile)
            statecommand = " -loadvm " + statefile_path
        
        if self.currimage_settings.customoptions != "":
            customoptions = " " + self.currimage_settings.customoptions      

        bootdriveletter = self.currimage_settings.boot_driveletter
        monitor_command = " -monitor pty"
        netcommand = self.qemu_network_command()


        basecommand = self.qemubin + machinecommand + memorycommand + statecommand + snapsot_command + localtime_command + " " + self.currimage_settings.kqemu_param + netcommand + usb_command + vgacommand + novgacommand + fullscreencommand + soundhwcommand + vnc_command + customoptions + cdrcommand + hda_command + hdb_command + hdc_command + hdd_command + fda_command + fdb_command
        self.basecommand = basecommand
        #print "qemu run mode: " + str(rmode)
        self.loadvm_command = self.qemubin + machinecommand + memorycommand + statecommand + snapsot_command + localtime_command + " " + self.currimage_settings.kqemu_param + netcommand + usb_command + vgacommand + novgacommand + fullscreencommand + soundhwcommand + vnc_command + customoptions + cdrcommand + hda_command + hdb_command + hdc_command + hdd_command + fda_command + fdb_command

        managed_command = basecommand + monitor_command
        unmanaged_command = basecommand
        
        if self.currimage_settings.dlinuxboot == True:
            kernelparam = ""
            if self.currimage_settings.kernelcmd != "" and self.currimage_settings.kernelcmd != None:
                kernelparam = " -append " + self.currimage_settings.kernelcmd
                #print "kernelimage: " + str(self.currimage_settings.kernelimage)
                #time.sleep(1)
            
            initrdcommand = ""
            if self.currimage_settings.initrd != "" and self.currimage_settings.initrd != None:
                initrdcommand = " -initrd " + str(self.currimage_settings.initrd)
                
            bootcommand =  lambda x : x + " -kernel " + str(self.currimage_settings.kernelimage) + str(initrdcommand) + str(kernelparam)
        else:
            bootcommand =  lambda x : x + " -" + self.currimage_settings.bootmedium  + " " + self.currentimage + " -boot " + bootdriveletter
        
        try:
            if rmode== "Managed":
                qemucommand = bootcommand(managed_command)
            else:
                qemucommand = bootcommand(unmanaged_command)
        except:
            qemucommand = bootcommand(unmanaged_command)
        #print "Qemu command1: " + str(qemucommand)
        return qemucommand
    
    ## Function to launch Qemu
    def run_qemu(self, widget):
        runmode = self.qemu_runmode()
        
        if widget.name != "toolbutton_runstate":
            self.statefile = None
        print "qemu called with command: " + str(self.qemu_command(runmode))
        #print "from widget: " + str(widget.name)
        #print "-------------------------Run Qemu--------------------------------------"
        if runmode == "Managed":
            command = self.qemu_command(runmode)
            child = popen2.Popen3(command, 1)
            child_stdin, child_stdout, child_stderr = child.tochild, child.fromchild, child.childerr
            child_stdin.close()
            pid = str(child.pid)
            print "started qemu with pid: " + str(pid)

            qemumonout = QemuGetPts(pid, child_stdout, child_stderr)
            qemumonout.start()   
            time.sleep(0.3)
            maxtry = 30
            trycount = 1
            #print "Looking for the launched pts terminal..."
            errormesg = _("No Message returned")
            while trycount < maxtry:
                try:
                    self.currentpts = qemumonout.currentpts
                    #print "Used pts device: " + str(self.currentpts)
                    break
                except:
                    try:
                        
                        errormesg = qemumonout.errormessage
                    except:
                        pass
                    #print "waiting for pts answer " + str(trycount) + " " + str(maxtry)
                    time.sleep(0.3)
                    trycount = trycount+1
            #print "vergleich " + str(trycount) + " " + str(maxtry)                
            if trycount == maxtry:
                print "timeout!"
                dialogtitle = _('Error!')
                dialogtext = _('Qemu exited with Error message:\n')
                dialog_markup = "<b>" + dialogtext + "</b>" + errormesg
                errordialog = Dialog("error", dialog_markup, dialogtitle)
            else:    
                selected_job_displaymode = "default"
                if self.currimage_settings.usevnc == True:
                    selected_job_displaymode = "vnc"
                elif self.currimage_settings.novga == True:
                    selected_job_displaymode = "novga"                
                ljob = [pid, self.currentpts, self.currentimage, selected_job_displaymode, 1]
                wjob = [pid, self.currentpts, self.currentimage]
    
                if self.jobwatcher.status == "stopped":
                    self.jobwatcher = QemuWatcher(self.joblist.remove_job2)
                self.joblist.add_job(ljob)
                self.jobwatcher.joblist.append(wjob)
                if self.jobwatcher.status == "stopped":
                    self.jobwatcher.start()
                if self.settings["controlmon_autostart"] == True:
                    time.sleep(0.3)
                    if os.path.exists(self.currentpts):
                        self.selected_job = [self.currentimage, pid, self.currentpts, selected_job_displaymode, 1]
                        self.selected_job = self.bookmark_job(self.selected_job)
                        print "JOB: " + str(self.selected_job)
                        self.run_jobmanager()
                    else:
                        errormesg = copy(str(child_stderr.readlines(1024)).strip('[]').strip())
                        dialogtitle = _('Error!')
                        dialogtext = _('Qemu exited with Error message:\n')
                        dialog_markup = "<b>" + dialogtext + "</b>\n" + errormesg
                        errordialog = Dialog("error", dialog_markup, dialogtitle)
     
        elif runmode == "Unmanaged":  
            os.popen(self.qemu_command(runmode) + "&")
        elif runmode == "Terminal":  
            os.popen("xterm -hold -e %s &" %self.qemu_command(runmode))
        
#########################################
## Functions for the settings dialog        
    def create_model_cdromdrives(self, data):
        store = gtk.ListStore(gobject.TYPE_STRING,
                          gobject.TYPE_STRING)
        for item in data:
            iter = store.append()
            itempath = item[0]
            itemname = item[1]
            store.set(iter, 0, item[0], 1, item[1])
        return store    

    def add_columns_cdromdrives(self, treeview, model):
    
        renderer = gtk.CellRendererToggle()
        column = gtk.TreeViewColumn(_('Name'), gtk.CellRendererText(), text=0)
        column.set_resizable(True)
        column2 = gtk.TreeViewColumn(_('Device'), gtk.CellRendererText(), text=1)
        column2.set_resizable(True)
        treeview.append_column(column)   
        treeview.append_column(column2) 
        
    def check_haveconfig(self, bootimage):
        haveconfig = False
        try:
            if self.image_settings[bootimage]:
                #print "Config for " + str(bootimage) + " found"
                haveconfig = True
            else:
                #print "No config for " + str(bootimage) + " found"
                haveconfig = False
        except:       
            #print "No config for " + str(bootimage) + " found"
            haveconfig = False
        return haveconfig
        
    def load_bookmarks(self, *widget):
        self.bookmarks = []
        try:
            outdata = getfromfile("bookmarks")
            if outdata  == { } or outdata == "":
                bookmarks = []
            else:
                bookmarks = outdata
        except:
            bookmarks = []
        if bookmarks != []:
            for entry in bookmarks:
               
                try:
                    if len(entry) == 4:
                        itemicon = entry[0]
                        itemname = entry[1]
                        itempath = entry[2] 
                        isoldconfig = True                 
                    elif len(entry) == 3 or len(entry) == 2:
                        itemicon = self.default_machine_icon
                        itemname = entry[0]
                        itempath = entry[1] 
                        isoldconfig = True
                except:
                    itemicon = entry.icon
                    itemname = entry.name
                    itempath = entry.path             
                    isoldconfig = False 
                haveconfig = self.check_haveconfig(itempath)
                
                if isoldconfig == True:
                    self.bookmarks.append((itemicon, itemname, itempath, haveconfig))
                else:
                    entry.haveconfig = haveconfig
                    self.bookmarks.append(entry)
        try:
            model = self.treeview_bookmarks.get_model()
            model.clear()
            model = self.create_model_bookmarks(self.bookmarks)
            self.treeview_bookmarks.set_model( model)
        except:
            model = self.create_model_bookmarks(self.bookmarks)
            self.treeview_bookmarks.set_model( model)
            self.add_columns_bookmarks( self.treeview_bookmarks, model)
   
    def create_model_bookmarks(self, data):
        store = gtk.ListStore(gtk.gdk.Pixbuf,
                          gobject.TYPE_STRING,
                          gobject.TYPE_STRING,
                          gtk.gdk.Pixbuf)
        
        config_icon = gtk.Image()
        for item in data:
            iter = store.append()
            try:
                if len(item) == 4:
                    itemicon = item[0]
                    itemname = item[1]
                    itempath = item[2]
                    haveconfig = item[3] 
                    isoldconfig = True                 
                elif len(item) == 3:
                    itemicon = self.default_machine_icon
                    itemname = item[0]
                    itempath = item[1] 
                    haveconfig = item[2]
                    isoldconfig = True               
            except:
                itemicon = str(item.icon)
                itemname = str(item.name)
                itempath = str(item.path)
                haveconfig = item.haveconfig 
                modeldata = item.get_modeldata() 
                #print "modeldate: " + str(modeldata)
                isoldconfig = False
                #print "load new bookmark"  
            #print "haveconfig: " + str(haveconfig)
            system_icon = None
            try:
                system_icon = gtk.gdk.pixbuf_new_from_file(itemicon)
            except:
                try:
                    icondir = icondirs[0]
                    system_icon = gtk.gdk.pixbuf_new_from_file(icondir + "/default.png")
                except:
                    pass
            
            if isoldconfig == False:
                config_icon = modeldata[3]
            else:
                if haveconfig == True:
                    config_icon = gtk.gdk.pixbuf_new_from_file(self.pixmapdir + '/gtk-yes.png')
                else:
                    config_icon = gtk.gdk.pixbuf_new_from_file(self.pixmapdir + '/gtk-no.png')
            #print "haveconfig: " + str(haveconfig) + " icon: " + str(config_icon) + "for item: " + str(itempath)
            try:
                pwidth = system_icon.get_width()
                pheight = system_icon.get_height()
                pfact = float(pwidth) / float(pheight)
                oheight = float(self.default_iconsize) / pfact
                oheight = self.default_iconsize / pfact
                icon = system_icon.scale_simple(int(self.default_iconsize), int(oheight), 3)            
            
                store.set(iter, 0, icon, 1, itemname, 2, itempath, 3, config_icon)
            except:
                store.set(iter, 0, system_icon, 1, itemname, 2, itempath, 3, config_icon)
        return store  
   
    def add_columns_bookmarks(self, treeview, model):
    
        renderer = gtk.CellRendererToggle()
        column_icon = gtk.TreeViewColumn(_('Icon'), gtk.CellRendererPixbuf(), pixbuf=0)
        column = gtk.TreeViewColumn(_('Name'), gtk.CellRendererText(), text=1)
        column.set_resizable(True)
        column2 = gtk.TreeViewColumn(_('Boot image'), gtk.CellRendererText(), text=2)
        column2.set_resizable(True)
        column3 = gtk.TreeViewColumn(_('configured'), gtk.CellRendererPixbuf(), pixbuf=3)
        column3.set_resizable(True)
        treeview.append_column(column_icon)
        treeview.append_column(column)   
        treeview.append_column(column2)  
        treeview.append_column(column3)
 
    def load_joblist(self, *widget):
        self.joblist = Joblist(self.treeview_joblist)
        self.joblist.jobmanagers = self.jobmanagers
        if os.path.exists(self.configdir + "/joblist"):
            self.joblist.load_joblist()
        if self.joblist.joblist_data and self.joblist.joblist_data != []:
            jc = 0    
            
            #self.jobwatcher = QemuWatcher(self.joblist.remove_job2)            
            for job in self.joblist.joblist_data:
                #time.sleep(0.1)
                
                pid = job[0]
                pts = job[1]
                path = job[2]
                displaymode = job[3]
                ljob = [pid, pts, path]
                wjob = [pid, pts, path]
                chcommand = "ps -e | grep " + str(pid)
                #print "chcommand: " + chcommand
                checkpid = os.popen("ps -e | grep " + str(pid)).readlines()
                #print "checking for pid " + str(pid) + ": " + str(checkpid)
                if os.path.exists(pts) and checkpid != []:
                    if self.jobwatcher.status == "stopped":
                        self.jobwatcher = QemuWatcher(self.joblist.remove_job2)
                    self.jobwatcher.joblist.append(wjob)
                    if self.jobwatcher.status == "stopped":
                        self.jobwatcher.start()
                    time.sleep(0.1)
                else:
                    #print "job is not running, remove from list"
                    self.joblist.remove_job2(ljob)
                    try:
                        jindex = self.jobwatcher.joblist.index(wjob)
                        del self.jobwatcher.joblist[jindex]
                        #print "removed from watcher daemon"
                    except:
                        #print "not in list"
                        pass
                jc = jc+1
       
    def bookmark_selection(self, widget):

        model, iter = widget.get_selection().get_selected()
        row = model[iter]
        item = row[2]
        #print "selected bookmark: %s" %item
        self.set_currentimage(item)  
        
    ## Checks if the given job has a bookmark. If true, append the name as 6. list entry
    ## job = a list containing job informations    
    def bookmark_job(self, job):
        image = job[0]
        pid = job[1]
        pts = job[2]
        runstat = job[3]
        name = image
        if self.bookmarks != []:                
            for bookmark in self.bookmarks:
                if image == bookmark.path:
                    #print "Bookmark: " + str(bookmark.name)
                    name = bookmark.name
        outjob = job
        outjob.append(name)
        return outjob           
        
    def add_bookmark(self, widget, *event):
        GoOn = False
        mode = "add"
        menuitem = widget.get_name()
        #print "menuitem: " + str(menuitem)
        if menuitem == "item_bookmark" or menuitem == "toolbutton_add_bookmark":
            mode = "add"
            response = self.add_bookmark_response
        if menuitem == "item_edit_bookmark":
            mode = "edit"
            response = self.edit_bookmark_response
            
        self.dialog_bookmarks = gtk.glade.XML(self.gladefile, "dialog_bookmarks", APP)
        self.dialog_bookmarks_widget = self.dialog_bookmarks.get_widget("dialog_bookmarks")
        if mode == "edit":
            try:
                self.dialog_bookmarks_widget.set_title("Edit selected Machine entry")
                model, iter = self.treeview_bookmarks.get_selection().get_selected()
                counter = int(model.get_string_from_iter(iter))
                #print "counter: " + str(counter)
                self.bookmark_tmpitem = self.bookmarks[counter]
                try:
                    name = self.bookmark_tmpitem.name
                    inpath = self.bookmark_tmpitem.path
                    icon = self.bookmark_tmpitem.icon
                except:
                    name = self.bookmark_tmpitem[1]
                    inpath = self.bookmark_tmpitem[2]
                    icon = path.abspath(self.bookmark_tmpitem[0])  
                    if icon == "":
                        icon = path.abspath(self.default_machine_icon)
                GoOn = True             
            except:
                print "No machine selected"
                GoOn = False        
        else:
            try:
                inpath = self.currentimage
                GoOn = True
            except:
                print "No item selected"
                GoOn = False
        if GoOn != False:
            Name_field = self.dialog_bookmarks.get_widget("entry_bookmark_name")
            Path_field = self.dialog_bookmarks.get_widget("entry_bookmark_path")
            filechooserbutton_machine_icon = self.dialog_bookmarks.get_widget("filechooserbutton_machine_icon")
            image_iconpreview = self.dialog_bookmarks.get_widget("image_iconpreview")
            
            Path_field.set_text(inpath)
            if mode == "edit":
                Name_field.set_text(name)
                iconpath = str(icon)                
            else:
                iconpath = path.abspath(self.default_machine_icon)
            filechooserbutton_machine_icon.set_filename(iconpath)
            try:
                preview_icon = gtk.gdk.pixbuf_new_from_file(iconpath)
                preview_icon2 = preview_icon.scale_simple(32, 32, 3)
                image_iconpreview.set_from_pixbuf(preview_icon2)
            except:
                pass
            Name = Name_field.get_text()
            Path = Path_field.get_text()
            
            dic = {"on_dialog_bookmarks_response" : response,
                   "on_filechooserbutton_machine_icon_selection_changed" : self.update_bm_iconprev
            }
            self.dialog_bookmarks.signal_autoconnect(dic)
            
    def update_bm_iconprev(self, widget):
        filechooserbutton_machine_icon = self.dialog_bookmarks.get_widget("filechooserbutton_machine_icon")
        image_iconpreview = self.dialog_bookmarks.get_widget("image_iconpreview")
        entry_icon = str(filechooserbutton_machine_icon.get_filename())
        if entry_icon != 'None':
            preview_icon = gtk.gdk.pixbuf_new_from_file(entry_icon)
            preview_icon2 = preview_icon.scale_simple(32, 32, 3)
            image_iconpreview.set_from_pixbuf(preview_icon2)

    def add_bookmark_response(self, widget, response, *entry):
        entryname = self.dialog_bookmarks.get_widget("entry_bookmark_name")
        entrypath = self.dialog_bookmarks.get_widget("entry_bookmark_path")
        filechooserbutton_machine_icon = self.dialog_bookmarks.get_widget("filechooserbutton_machine_icon")

        entry_icon = str(filechooserbutton_machine_icon.get_filename())
        entry_icon_pixbuf = gtk.gdk.pixbuf_new_from_file(entry_icon)
        
        entry_name = str(entryname.get_text())
        entry_path = str(entrypath.get_text())
        haveconfig = str(self.check_haveconfig(entry_path))
        if haveconfig == True:
            config_icon = gtk.gdk.pixbuf_new_from_file(self.pixmapdir + '/gtk-yes.png')
        else:
            config_icon = gtk.gdk.pixbuf_new_from_file(self.pixmapdir + '/gtk-no.png')
        #print 'entryname: ' + entry_name
        #print 'entrypath: ' + entry_path
        if response == gtk.RESPONSE_OK:  
            newbookmark = Bookmark()
            newbookmark.icon = entry_icon
            newbookmark.name = entry_name
            newbookmark.path = entry_path
            newbookmark.haveconfig = haveconfig
            try:
                self.bookmarks.append(newbookmark)
            except:
                self.bookmarks = [(newbookmark)]
            self.storetofile(self.bookmarks, "bookmarks")
            self.refresh_bookmarks()
            widget.destroy()
            return entryname                  
                
        elif response == gtk.RESPONSE_CANCEL:
            widget.destroy()   
            return None               
        return None                
                
    def edit_bookmark_response(self, widget, response, *entry):
        #print 'widget: ' + str(widget)
        #print 'response: ' + str(response)
        entryname = self.dialog_bookmarks.get_widget("entry_bookmark_name")
        entrypath = self.dialog_bookmarks.get_widget("entry_bookmark_path")
        #filechooserbutton_machine_icon = self.dialog_bookmarks.get_widget( "filechooserbutton_machine_icon")
        filechooserbutton_machine_icon = self.dialog_bookmarks.get_widget("filechooserbutton_machine_icon")
        fname = filechooserbutton_machine_icon.get_filename()
        entry_icon = str(fname)
        entry_icon_pixbuf = gtk.gdk.pixbuf_new_from_file(entry_icon)
        entry_name = str(entryname.get_text())
        entry_path = str(entrypath.get_text())
        haveconfig = str(self.check_haveconfig(entry_path))
        if haveconfig == True:
            config_icon = gtk.gdk.pixbuf_new_from_file(self.pixmapdir + '/gtk-yes.png')
        else:
            config_icon = gtk.gdk.pixbuf_new_from_file(self.pixmapdir + '/gtk-no.png')
        #print 'entryname: ' + entry_name
        #print 'entrypath: ' + entry_path
        if response == gtk.RESPONSE_OK:
            
            model, iter = self.treeview_bookmarks.get_selection().get_selected()
            counter = int(model.get_string_from_iter(iter))
                            
            editbookmark = Bookmark()
            editbookmark.icon = entry_icon
            editbookmark.name = entry_name
            editbookmark.path = entry_path
            editbookmark.haveconfig = haveconfig
            self.bookmarks[counter] = editbookmark
            self.storetofile(self.bookmarks, "bookmarks")
            self.refresh_bookmarks()
            widget.destroy()
            return entryname      
        elif response == gtk.RESPONSE_CANCEL:
            widget.destroy()   
            return None               
        return None
        
    def remove_bookmark(self, widget):
        entry_path = self.selected_bookmark
        #print 'widget: ' + str(widget)
        #print 'entry: ' + str(entry_path)
        model, iter = self.treeview_bookmarks.get_selection().get_selected()
        counter = int(model.get_string_from_iter(iter))
        #print "bookmark to remove: " + str(self.bookmarks[counter].name)
        #print "bookmark ct to remove: " + str(counter)
        del self.bookmarks[counter]
        self.storetofile(self.bookmarks, "bookmarks")
        self.refresh_bookmarks()
        
    def sort_bookmarks(self, *widget):
        bookmarks = []
        #time.sleep(0.1)
        model = self.treeview_bookmarks.get_model() 
        for row in model:
            itemicon = row[0]
            itemname = row[1]
            itempath = row[2]
            haveconfig = self.check_haveconfig(itempath)
            newbookmark = Bookmark()
            iconpath = self.search_bookmark(itempath, "icon")
            newbookmark.icon = iconpath
            newbookmark.name = itemname
            newbookmark.path = itempath
            newbookmark.haveconfig = haveconfig
            try:
                bookmarks.append(newbookmark)
            except:
                bookmarks = [(newbookmark)]
        self.bookmarks = bookmarks                       
        self.storetofile(self.bookmarks, "bookmarks")

    def search_bookmarklist(self, widget, searchstring, returnval):
        ct = 0
        outct = "None"
        bmmodel = widget.get_model()
        #print "searchstring: " + str(searchstring)
        for row in bmmodel:
            #print "row: " + str(row[0]) + " - " + str(row[1]) + " - " + str(row[2])
            if searchstring == row[2]:
                if returnval == "index":
                    outct = ct
                elif returnval == "icon":
                    outct = row[0]                   
                else:
                    outct = row[1]
            ct = ct + 1      
        #print "search bookmark result: " + str(outct)            
        return outct     
    def search_bookmark(self, searchstring, returnval):
        ct = 0
        outct = None
        for row in self.bookmarks:
            #print "row: " + str(row[0]) + " - " + str(row[1]) + " - " + str(row[2])
            if searchstring == row.path:
                if returnval == "index":
                    outct = ct
                elif returnval == "icon":
                    outct = row.icon                   
                else:
                    outct = row.name
                
            ct = ct + 1      
        #print "search bookmark result: " + str(outct)            
        return outct             
    def toggle_storebuttons(self, state, caller=None):
        #gtk.threads_enter()
        self.button_apply_imagesettings.set_sensitive(state) 
        self.button_restore_imagesettings.set_sensitive(state)
        #gtk.threads_leave()
        #print "Storebuttons toggled"
        #if caller != None:
        #    print "callerfunction: " + caller
         
    ## The main function to load current selected image and fetch the settings for it
    ## Its allways called from the widget 'entry_current_image'!
    def select_image_from_widget(self, widget):
        currentpath = widget.get_text()
        self.select_image_from_path(currentpath)
    
    def select_image_from_path(self, srcpath):
        current = srcpath
        if os.path.exists(current) and os.path.isfile(path.abspath(current)):  
            #self.restoreaction = True
            #print "entered path: " + str(current)
            isbookmark = False
            for bookmark in self.bookmarks:
                try:
                    if current == bookmark[1]:
                        #print "Bookmark: " + str(bookmark[1])
                        isbookmark = True
                        break
                except:
                    if current == bookmark.path:
                        #print "Bookmark: " + str(bookmark[1])
                        isbookmark = True
                        break                   
            #self.toolbutton_remove_bookmark.set_sensitive( isbookmark)      
            self.notebook_imgsettings.set_sensitive(True)
            self.currentimage = current
            self.toolbutton_add_bookmark.set_sensitive(True)
            #print "selected current image from path"
            self.restore_emu_config()
            #print "go on!"
            #self.restoreaction = False
            #self.toggle_storebuttons(False, "select_image_from_path")
        else:
            self.label_imgsettings.set_markup( _('<b>No image selected</b>'))
            self.activate_launchbuttons(False)
            self.notebook_imgsettings.set_sensitive(False)  
            self.toolbutton_add_bookmark.set_sensitive(False) 
            print "No image selected from path" 
        
    def pop_up_bookmarks(self, widget, event):    
        if event.button == 3:
            #print "bookmark_popup"
            sel = self.treeview_bookmarks.get_selection().get_selected()
            #print "iter: " + str(sel[1])
            model = self.treeview_bookmarks.get_model()
            try:            
                self.selected_bookmark = model.get_value(sel[1], 2)
                self.menu_popup_bookmarks.popup( None, None, None, 3, 0)
            except:
                print "no bookmark selected"
        else:
            pass                     
            
    #def cdrive_selection(self, widget, path, *view_column):
    def cdrive_selection(self, widget):
        model, iter = widget.get_selection().get_selected()
        row = model[iter]
        item = row[1]
        #print "item: %s" %item
        self.set_currentimage(item)
        
    def clear_filechooser(self, widget):
        #print "name: " + str(widget.name)
        #fcb = None
        wname = None
        if widget == self.button_eject_hda:
            wname = 'filechooserbutton_hda'
#            fcb = self.wTree[wname]
        elif widget == self.button_eject_hdb:
            wname = 'filechooserbutton_hdb'
            #fcb = self.filechooserbutton_hdb
        elif widget == self.button_eject_hdc:
            wname = 'filechooserbutton_hdc'
            #fcb = self.filechooserbutton_hdc 
        elif widget == self.button_eject_hdd:
            wname = 'filechooserbutton_hdd'
            #fcb = self.filechooserbutton_hdd 
        elif widget == self.button_eject_fda:
            wname = 'filechooserbutton_fda'
            fcb = self.filechooserbutton_fda
        elif widget == self.button_eject_fdb:
            wname = 'filechooserbutton_fdb'
            #fcb = self.filechooserbutton_fdb            
        elif widget == self.button_eject_imagecdrom:
            wname = 'filechooserbutton_imagecdrom'
            #fcb = self.filechooserbutton_imagecdrom
        else:
            #fcb = None
            wname = None
        if wname != None: 
            fcb = self.wTree.get_widget(wname)
            try:
                _hid = self.sighandlers[wname]
                fcb.handler_block(_hid)
            except:
                pass
               
            try:
                fcb.set_current_folder(self.settings["imagespath"])
            except:
                fcb.unselect_all()
            try:                
                fcb.handler_unblock(_hid)
            except:
                pass                
                   
    def pop_up_imagelist(self, widget, event):
        
        if event.button == 3:
            sel = self.isolist.get_selection().get_selected()
            model = self.isolist.get_model()
            try:
                self.selected_image = model.get_value(sel[1], 0)
                self.currentimage = self.selected_image
                self.menu_popup_imagelist.popup( None, None, None, 3, 0)
            except:
                print "no image selected"                
                
    def joblist_selection(self, widget):
        try:
            model, iter = widget.get_selection().get_selected()
            row = model[iter]
            selected_job_path = row[0]
            selected_job_pid = row[1]
            selected_job_pty = row[2]
            selected_job_displaymode = row[3]
            selected_job_runstat = row[4]
            #print "selected job : %s" %selected_job_pty
            self.selected_job = [selected_job_path, selected_job_pid, selected_job_pty, selected_job_displaymode, selected_job_runstat]
            self.selected_job = self.bookmark_job(self.selected_job)
            
            if selected_job_runstat == "1":
                self.item_cont_job.hide()
                self.item_stop_job.show()
                print "running"
            else:
                self.item_cont_job.show()
                self.item_stop_job.hide()
                print "stopped"
        except:
            print "no job selected" 
            
    def pause_job(self, widget):
        job = self.selected_job
        try:
            self.jobmanagers[job[1]].pause_job(widget)
            #print "jobmanager found, stopping through its method"
        except:
            self.joblist.pause_job(widget, job)
        
    def cont_job(self, widget):
        job = self.selected_job
        try:
            self.jobmanagers[job[1]].cont_job(widget)
            #print "jobmanager found, starting through its method"
        except:
            self.joblist.cont_job(widget, job)   

    def reset_job(self, widget):
        job = self.selected_job
        try:
            self.jobmanagers[job[1]].reset_job(widget)
            #print "jobmanager found, starting through its method"
        except:
            self.joblist.reset_job(widget, job)
            
    def stop_job(self, widget):
        job = self.selected_job
        print "stopped job!"
        self.joblist.stop_job(widget, job)               

    def pop_up_joblist(self, widget, event):
        try:
            selected_job_path = self.selected_job[0]
            selected_job_pid = self.selected_job[1]
            selected_job_pty = self.selected_job[2]
            selected_job_displaymode = self.selected_job[3]
            selected_job_runstat = self.selected_job[4]
            self.currentpts = self.selected_job[2]
            
            if event.type == gtk.gdk._2BUTTON_PRESS:
                self.run_jobmanager(widget, self.selected_job)
            elif event.button == 3:
                self.menu_popup_joblist.popup(None, None, None, 3, 0)            
        except:
            print "no job selected"                    
    
     
    def show_infomessage(self, *widget):
        image = self.selected_image
        info = self.fetch_image_info(image)
        title = _('image info')
        self.init_messages_dialog(info, title)
        
    def show_infomessage_b(self, *widget):
        image = self.selected_bookmark
        info = self.fetch_image_info(image)
        title = _('image info')
        self.init_messages_dialog(info, title)      

    def fetch_settings(self, *widget):
        self.qemubin = self.settings["qemuapps"][0][1]

        if self.settings.has_key("imagespath"):
            self.imagespath = self.settings["imagespath"]

        if self.settings.has_key("settings_autoshow"):
            self.settings_autoshow = self.settings["settings_autoshow"]

        if self.settings.has_key("settings_autohide"):        
            self.settings_autohide = self.settings["settings_autohide"]

        if self.settings.has_key("default_iconsize"):           
            self.default_iconsize = self.settings["default_iconsize"]

        self.get_soundhw()
            
    def get_soundhw(self, *event):
        qemucommand = self.qemubin
        prog = os.popen(str(qemucommand) + " -soundhw ?")
        output = prog.readlines()
        output2 = []
        self.audio_devlist = []
        fc = 0
        for f in output:
            f = string.strip(f)
            if f != "":
                output2.append(f)
                fc = fc+1
        for f in output2:
            if f != output2[0] and f != output2[fc-1]:
                
                devdata = f.split(" ", 1)
                devdata = filter(None, devdata)
                devdata[0] = string.strip(devdata[0])
                devdata[1] = string.strip(devdata[1])
                self.audio_devlist.append(devdata) 
    
    def refresh_soundhw(self, *event):
        audiomodel = self.combobox_audiohw.get_model()
        audiomodel.clear()
        audiomodel.append(row=("None",))
        for dev in self.audio_devlist:
            device = dev[0]
            name = dev[1]
            #print "dev: " + name + " - " + device
            audiomodel.append(row=(name,))
        self.combobox_audiohw.set_model(audiomodel)      
        self.combobox_audiohw.set_active(0)  
        
    ## Fetch the available machines for the selected emu system
    def get_machine(self, *event):
        #print "get machine"  
        #print "qemu system: " + str(self.qemubin)
        prog = os.popen(str(self.qemubin) + " -M ?")
        output = prog.readlines()
        output2 = []
        self.machineslist = []
        fc = 0
        for f in output:
            #print "count machines: " + str(fc)
            f = string.strip(f)
            #print "f: " + str(f)
            if f != "":
                output2.append(f)
                fc = fc+1
        for f in output2:
            if f != output2[0]:
                devdata = f.split(" ", 1)
                devdata = filter(None, devdata)
                devdata[0] = string.strip(devdata[0])
                devdata[1] = string.strip(devdata[1])
                self.machineslist.append(devdata)   
            
    def refresh_machineslist(self, *event):
        machinemodel = self.combobox_machinetype.get_model()
        machinemodel.clear()
        for dev in self.machineslist:
            device = str(dev[0])
            name = str(dev[1])
            #print "dev: " + name + " - " + device
            #print str(dev[0]) + " - " + str(dev[1])
            machinemodel.append(row=(name,))
            self.combobox_machinetype.set_model(machinemodel)  
        self.combobox_machinetype.set_active(0)     
                
    def refresh_ilist(self, *widget): 
        self.ilist_data = []   
        ilistmodel = self.isolist.get_model() 
        ilistmodel.clear()
        #print "refresh ilist imagespath: " + self.settings["imagespath"]
        data=os.listdir(self.settings["imagespath"])
        data.sort()
        for entry in data:
            path = str(self.settings["imagespath"]) + "/" + str(entry)
            #print "path: " + path
            haveconfig = self.check_haveconfig(path)
            #print "haveconfig: " + str(haveconfig)
            self.ilist_data.append((path, haveconfig))
        
        ilistmodel = self.create_model_ilist(self.ilist_data) 
        #print "refresh ilistmodel: " + str(ilistmodel)   
        setmod = self.isolist.set_model(ilistmodel)
        #print "set ilistmodel: " + str(setmod)
        
    def refresh_devlist(self, *widget): 
        devlist_data = []   
        model = self.treeview_cdrives.get_model() 
        model.clear()
        model = self.create_model_cdromdrives(self.settings["cdromdrives"]) 
        setmod = self.treeview_cdrives.set_model(model)
        
    def refresh_bookmarks(self, *widget): 
        self.load_bookmarks()
      

            
## Open settings dialog

    def open_settings(self, *widget):
        self.settingsdialog = qml_configuration.Configuration(self.settings_callback)
        
    def settings_callback(self):
        self.settings = load_settings()
        print "Get settings from settings callback function"
        for data in self.settings["cdromdrives"]:
            print "device: " + str(data)
        self.fetch_settings()
        #gtk.threads_enter()
        self.refresh_ilist()
        self.refresh_devlist()
        self.toggle_settings_autoshow(self.settings_autoshow)
        self.toggle_settings_autohide(self.settings_autohide)
        #gtk.threads_leave()
        try:
            qemulator.refresh_bookmarks()
        except:
            pass
    
    def create_model_qemupaths(self, data):
        store = gtk.ListStore(gobject.TYPE_STRING,
                          gobject.TYPE_STRING)
        for item in data:
            iter = store.append()
            itempath = item[0]
            itemname = item[1]
            #print itempath
            #print itemname
            store.set(iter, 0, item[0], 1, item[1])
        return store        
        
    def add_columns_qemupaths(self, treeview, model):
    
        renderer = gtk.CellRendererToggle()
        column = gtk.TreeViewColumn('Name', gtk.CellRendererText(), text=0)
        column2 = gtk.TreeViewColumn('Path', gtk.CellRendererText(), text=1)
        treeview.append_column(column)   
        treeview.append_column(column2)           
    
    def settings_get_qemupath(self, widget, path, view_column):

        model = widget.get_model()
        row = model[path]
        item = row[0]
        #print "item: %s" %item
        self.settings_set_currentqemupath(row)    
        
    def settings_set_currentqemupath(self, device_list):
    
            nameinput = self.dialog_settings.get_widget("entry_settings_qemuname")
            devinput = self.dialog_settings.get_widget("entry_settings_qemupath")
            
            nameinput.set_text(device_list[0])
            devinput.set_text(device_list[1])  
    
    def settings_add_qemupath(self, widget):
            #print "new qemupath"
            name = self.dialog_settings.get_widget("entry_settings_qemuname").get_text()
            path = self.dialog_settings.get_widget("entry_settings_qemupath").get_text()
            qemulist = self.dialog_settings.get_widget("treeview_qemupaths")
            model = qemulist.get_model()
            model.append(row=(name, path))
            #print "qemuapp: " + name + " : " + path
            
            try:
                self.settings["qemuapps"].append((name, path))
                #self.bookmarks.append((name, entry_path))
            except:
                self.settings["qemuapps"] = [(name, path)]    
                
    def settings_remove_qemupath(self, widget):
        qemulist = self.dialog_settings.get_widget("treeview_qemupaths")
        name = self.dialog_settings.get_widget("entry_settings_qemuname").get_text()
        path = self.dialog_settings.get_widget("entry_settings_qemupath").get_text()
        #print 'name: ' + str(name)
        #print 'path: ' + str(path)
        entryindex = self.settings_search_cdromdevice(qemulist, str(path), "index")
        entry_name = name
        
        if entryindex != "None":
            model = qemulist.get_model()
            selectIter = model.get_iter(entryindex)
            #print str(selectIter)
            #print "entryindex" + str(entryindex)
            model.remove(selectIter)
            #self.bookmarks.remove((entry_name, entry_path))
            self.settings["qemuapps"].remove((name, path))
            #self.toolbutton_remove_bookmark.set_sensitive(False)
            self.dialog_settings.get_widget("entry_settings_qemuname").set_text("")
            self.dialog_settings.get_widget("entry_settings_qemupath").set_text("")
        else:
            print "no enty " + str(name)  + " found"
        return str(entry_name) 
    
    def toggle_dlinuxboot(self, widget):
        state = widget.get_active()
        if state == True:
            self.frame_dlinuxboot.show()
            self.button_hide_dlinuxboot.show()
            self.button_hide_dlinuxboot.set_label(_("Hide settings"))
            self.hbox_imagedevice.hide()
            self.filechooserbutton_hda.set_filename(str(self.currentimage))
            self.filechooserbutton_hda.set_sensitive(True)
        else:
            self.frame_dlinuxboot.hide()
            self.button_hide_dlinuxboot.hide()
            self.hbox_imagedevice.show()
            self.filechooserbutton_hda.unselect_all()
            #print "hda image: " + str(self.image_settings[self.currentimage].hda_image)
            if self.currimage_settings.bootmedium == "hda":
                self.filechooserbutton_hda.unselect_all()
                self.filechooserbutton_hda.set_sensitive(False)
            else:
                if self.currentimage == self.image_settings[self.currentimage].hda_image:
                    self.filechooserbutton_hda.unselect_all()
                elif self.image_settings[self.currentimage].hda_image != None and self.image_settings[self.currentimage].hda_image != 'None':
                    self.filechooserbutton_hda.set_filename(self.image_settings[self.currentimage].hda_image)
                   
        #self.vpaned_mainwindow.set_position(900)
        self.set_dlinuxboot()
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "toggle_dlinuxboot")        
        
    def toggle_hide_dlinuxboot(self, widget):
        if self.frame_dlinuxboot.get_property('visible'):
            self.frame_dlinuxboot.hide()
            widget.set_label(_("Show settings"))
        else:
            self.frame_dlinuxboot.show()
            widget.set_label(_("Hide settings"))
        #self.vpaned_mainwindow.set_position(900)
    
         
    def init_about(self, widget):
        donateurl = "http://qemulator.createweb.de/index.php4?aliasdoc=210"
        self.dialog_about = gtk.glade.XML(self.gladefile, "dialog_about", APP)
        self.dialog_about_widget = self.dialog_about.get_widget("dialog_about")
   
        logo_file = None
        logo_file = self.sharedir + "/logo3.png"
        #print "Logo: " + logo_file
        logopixbuf = gtk.gdk.pixbuf_new_from_file(logo_file)
        #self.logo_mainwindow.set_from_file(logo_file)           
        self.dialog_about_widget.set_logo(logopixbuf)
        hurl = "http://qemulator.createweb.de"
        wlabel = "qemulator home"
        gtk.about_dialog_set_url_hook(self.open_website, hurl)
        dic = {"on_dialog_about_response" : self.dialog_about_response,
        }
        self.dialog_about.signal_autoconnect(dic)          

    def dialog_about_response(self, widget, response, *entry):    
        if response == gtk.RESPONSE_CLOSE:
            widget.destroy ()
            pass    
        if response == gtk.RESPONSE_APPLY:
            self.dodonate()
            pass          
        
    def dodonate(self, *widget):
        donateurl = "http://qemulator.createweb.de/index.php4?aliasdoc=210"
        #print "donation clicked: " + donateurl
        browser = Webbrowser()  
        browser.url = donateurl
        browser.target = "new window"
        browser.start()       
            
    def init_create_image(self, widget):
        
        self.dialog_create_image = gtk.glade.XML(self.gladefile, "dialog_create_image", APP)
        ## widgets of the new image dialog
        
        self.filechooserbutton_newimage_dest = self.dialog_create_image.get_widget("filechooserbutton_newimage_dest")
        self.entry_newimage_name = self.dialog_create_image.get_widget("entry_newimage_name")
        self.combobox_newimage_format = self.dialog_create_image.get_widget("combobox_newimage_format")
        self.spinbutton_newimage_size = self.dialog_create_image.get_widget("spinbutton_newimage_size")
        self.combobox_newimage_sizeunit = self.dialog_create_image.get_widget("combobox_newimage_sizeunit")
        self.button_create_image = self.dialog_create_image.get_widget("button_create_image")        
        
        self.newimagename = ""
        self.combobox_newimage_sizeunit.set_active(0)
        self.set_newimagesize()
        self.combobox_newimage_format.set_active(0)
        self.set_newimageformat()      
        self.filechooserbutton_newimage_dest.set_filename(self.imagespath)
        self.set_newimage_folder()
        dic = {"on_dialog_create_image_response" : self.dialog_create_image_response
        # handlers for the new image dialog
        , "on_filechooserbutton_newimage_dest_selection_changed" : self.set_newimage_folder
        , "on_entry_newimage_name_changed" : self.set_newimage_name
        , "on_combobox_newimage_format_changed" : self.set_newimageformat
        , "on_spinbutton_newimage_size_changed" : self.set_newimagesize
        , "on_combobox_newimage_sizeunit_changed" : self.set_newimagesize
        , "on_button_create_image_clicked" : self.create_newimage
        }
        self.dialog_create_image.signal_autoconnect(dic)  
        
    def set_newimage_folder(self, *widget):
        self.newimagepath = self.filechooserbutton_newimage_dest.get_filename()
        #print "path: " + self.newimagepath
        
    def set_newimage_name(self, *widget):
        self.newimagename = self.entry_newimage_name.get_text() 
        #print "name: " + self.newimagename    
        
    def set_newimageformat(self, *widget):
        #self.newimageformat = self.combobox_newimage_format.get_active()
        active = self.combobox_newimage_format.get_active()
        model = self.combobox_newimage_format.get_model()
        row = model[active]
        self.newimageformat = row[0]
        
    def set_newimagesize(self, *widget):
        sizevalue = self.spinbutton_newimage_size.get_value()
        #sizeunit = self.combobox_newimage_sizeunit.get_active()
        active = self.combobox_newimage_sizeunit.get_active()
        model = self.combobox_newimage_sizeunit.get_model()
        row = model[active]
        sizeunit = row[0]
        
        if sizeunit == "KB":
            self.newimage_size = str(int(sizevalue))
        elif sizeunit == "MB":
            self.newimage_size = str(int(sizevalue)) + "M"
        elif sizeunit == "GB":
            self.newimage_size = str(int(sizevalue)) + "G" 
        else:
            self.newimage_size = str(int(sizevalue))
        #print "size: " + self.newimage_size
        
    def create_newimage(self, widget):
        if self.newimagename != "":
            newimagetarget = self.newimagepath + "/" + self.newimagename
            if self.newimageformat != "Auto":
                formatcommand = "-f " + self.newimageformat
            else:
                formatcommand = ""
            command = "qemu-img create " + formatcommand + " " + newimagetarget + " " + self.newimage_size
            #print command
            runcreate = os.popen(command).read()
            message = command + "\n" + str(runcreate)
            #print str(message)
            self.init_messages_dialog(str(message))
            self.messagesbuffer.set_text(str(message))
            self.refresh_ilist()
        else:
            message = _('No name given for the new image!\nplease enter a name and try again.')
            self.init_messages_dialog(message)
            
    def dialog_create_image_response(self, widget, response, *entry):    
        if response == gtk.RESPONSE_CLOSE:
            widget.destroy ()
            pass 
            
    def init_messages_dialog(self, message, *title):
        self.dialog_messages = gtk.glade.XML(self.gladefile, "dialog_messages", APP)
        #if title:
        #    self.dialog_messages.set_title(title)
        self.textview_messages = self.dialog_messages.get_widget("textview_messages")
        self.messagesbuffer = self.textview_messages.get_buffer()
        self.messagesbuffer.set_text(message)
        
        dic = {"on_dialog_messages_response" : self.dialog_messages_response,        
        }
        self.dialog_messages.signal_autoconnect(dic)         
        
    def dialog_messages_response(self, widget, response, *entry):    
        if response == gtk.RESPONSE_CLOSE:
            widget.destroy ()
            pass   
        
    #def select_networkcard(self, widget, path, view_column):
    def select_networkcard(self, widget):
        sel = self.treeview_redirects.get_selection()
        sel.unselect_all()

    #def select_networkredir(self, widget, path, view_column):
    def select_networkredir(self, widget):
        sel = self.treeview_networkcards.get_selection()
        sel.unselect_all()

    def init_networksetup(self, widget):
        GoOn = False        
        button = widget.get_name()
        if button == "toolbutton_network_netcard_add":
            mode = "add"
            response = self.dialog_add_netcard_response
            GoOn = True
        #if button == "button_network_edit":
        #    mode = "edit"
        if button == "toolbutton_network_netcard_edit":
            mode = "edit_card"
            response = self.dialog_edit_netcard_response
        if button == "toolbutton_network_redir_edit":
            mode = "edit_redir"                        
            response = self.dialog_network_edit_redirect_response
        if mode == "edit": # depreciated
            try:
                model, iter = self.treeview_networkcards.get_selection().get_selected()
                counter = int(model.get_string_from_iter(iter))
                GoOn = True
            except:
                print "No network card selected"
                GoOn = False
                try:
                    model, iter = self.treeview_redirects.get_selection().get_selected()
                    counter = int(model.get_string_from_iter(iter))
                    self.init_network_redirect(widget)
                    GoOn = False
                except:
                    print "No redirect selected"
                    GoOn = False
        if mode == "edit_card":
            try:
                model, iter = self.treeview_networkcards.get_selection().get_selected()
                counter = int(model.get_string_from_iter(iter))
                GoOn = True
            except:
                print "No network card selected"
                GoOn = False
        if mode == "edit_redir":
            try:
                model, iter = self.treeview_redirects.get_selection().get_selected()
                counter = int(model.get_string_from_iter(iter))
                self.init_network_redirect(widget)
                GoOn = False
            except:
                print "No redirect selected"
                GoOn = False                                  
        if GoOn == True:
            self.tmpnetwork = qml_network.Network()
            self.dialog_newnetcard = gtk.glade.XML(self.gladefile, "dialog_newnetcard", APP)
            self.dialog_newnetcard_widget = self.dialog_newnetcard.get_widget("dialog_newnetcard")
            ## widgets of the new image dialog
            self.combobox_networktype = self.dialog_newnetcard.get_widget("combobox_networktype")
            self.combobox_networktype.set_active(0)
            
            self.entry_network_ip = self.dialog_newnetcard.get_widget("entry_network_ip")
            self.spinbutton_network_port = self.dialog_newnetcard.get_widget("spinbutton_network_port")
            self.entry_network_macaddr = self.dialog_newnetcard.get_widget("entry_network_macaddr")
            self.spinbutton_network_vlan = self.dialog_newnetcard.get_widget("spinbutton_network_vlan")
            self.entry_network_tuntapscript = self.dialog_newnetcard.get_widget("entry_network_tuntapscript")        
            self.button__network_open_tuntap_file = self.dialog_newnetcard.get_widget("button__network_open_tuntap_file")
            self.entry_network_ifacename = self.dialog_newnetcard.get_widget("entry_network_ifacename")
            self.spinbutton_network_filedescriptor = self.dialog_newnetcard.get_widget("spinbutton_network_filedescriptor")
            
            dic = {"on_combobox_networktype_changed" : self.select_networktype
            , "on_entry_network_ip_changed" : self.set_network_ip
            , "on_spinbutton_network_port_changed" : self.set_network_port
            , "on_entry_network_macaddr_changed" : self.set_network_mac
            , "on_spinbutton_network_vlan_changed" : self.set_network_vlan
            , "on_entry_network_tuntapscript_changed" : self.set_network_ttscript
            , "on_entry_network_ifacename_changed" : self.set_network_ifname
            , "on_spinbutton_network_filedescriptor_changed" : self.set_network_ttfiledesc
            , "on_dialog_newnetcard_response" : response
            }
            self.dialog_newnetcard.signal_autoconnect(dic)  
            self.select_networktype(self.combobox_networktype)
            
            if mode == "edit_card" or mode == "edit_redir":
                self.dialog_newnetcard_widget.set_title(_("Edit selected network card"))
                self.network_editcard = self.currimage_settings.networksetup[counter]
                self.tmpnetwork = self.network_editcard
                
                searchentry = self.search_combolist(self.combobox_networktype, self.network_editcard.ntype)
                if searchentry != "":
                    self.combobox_networktype.set_active(searchentry)
                
                self.entry_network_ip.set_text(str(self.network_editcard.ip))
                self.spinbutton_network_port.set_value(int(self.network_editcard.port))
                self.entry_network_macaddr.set_text(str(self.network_editcard.mac))
                self.spinbutton_network_vlan.set_value(int(self.network_editcard.vlan))
                self.entry_network_tuntapscript.set_text(str(self.network_editcard.ttscript))
                self.entry_network_ifacename.set_text(str(self.network_editcard.ifname))
                self.spinbutton_network_filedescriptor.set_text(str(self.network_editcard.ttfiledesc))
        
    def dialog_add_netcard_response(self, widget, response, *entry):    
        if response == gtk.RESPONSE_CANCEL:
            self.tmpnetwork = []
            widget.destroy ()
        if response == gtk.RESPONSE_OK:
            self.currimage_settings.networksetup.append(self.tmpnetwork)
            
            self.refresh_networkcards()
            self.tmpnetwork = []
            #print "Network device stored"
            widget.destroy ()
            self.toggle_storebuttons(True, "dialog_add_netcard_response")
            
    def dialog_edit_netcard_response(self, widget, response, *entry):    
        if response == gtk.RESPONSE_CANCEL:
            self.tmpnetwork = []
            widget.destroy ()
        if response == gtk.RESPONSE_OK:
            self.network_editcard = self.tmpnetwork
            self.tmpnetwork = []
            self.refresh_networkcards()
            #print "Network device changed"
            widget.destroy ()
            self.toggle_storebuttons(True, "dialog_edit_netcard_response")           
            
    def remove_netcard(self, widget):
        GoOn = True
        try:
            model, iter = self.treeview_networkcards.get_selection().get_selected()
            counter = int(model.get_string_from_iter(iter))
        except:
            print "No network card selected for remove"
            GoOn = False
            try:
                model, iter = self.treeview_redirects.get_selection().get_selected()
                counter = int(model.get_string_from_iter(iter))
                self.remove_redirect(widget)
                GoOn = False
            except:
                print "No redirect selected for remove"
                GoOn = False           
        if GoOn == True:
            ip = model.get_value(iter, 0)
            port = model.get_value(iter, 1)
            vlan = model.get_value(iter, 2)
            counter = int(model.get_string_from_iter(iter))
            del self.currimage_settings.networksetup[counter]
            self.refresh_networkcards()
            self.toggle_storebuttons(True, "remove_netcard")             
        
    def select_networktype(self, widget):
        active = widget.get_active()
        model = widget.get_model()
        row = model[active]
        item = row[0]
        self.tmpnetwork.ntype = item
        time.sleep(0.1)
        if item == "Open a TUN/TAP interface":
            self.entry_network_ip.set_sensitive(False)
            self.spinbutton_network_port.set_sensitive(False)
            self.entry_network_tuntapscript.set_sensitive(True)
            self.button__network_open_tuntap_file.set_sensitive(True)
            self.entry_network_ifacename.set_sensitive(True)
            self.spinbutton_network_filedescriptor.set_sensitive(False)
        elif item == "Use an already open TUN/TAP interface":
            self.entry_network_ip.set_sensitive(False)
            self.spinbutton_network_port.set_sensitive(False)
            self.entry_network_tuntapscript.set_sensitive(False)
            self.button__network_open_tuntap_file.set_sensitive(False)
            self.entry_network_ifacename.set_sensitive(False)
            self.spinbutton_network_filedescriptor.set_sensitive(True)
        elif item == "Open a listening TCP socket":
            self.entry_network_ip.set_sensitive(True)
            self.spinbutton_network_port.set_sensitive(True)
            self.entry_network_tuntapscript.set_sensitive(False)
            self.button__network_open_tuntap_file.set_sensitive(False)
            self.entry_network_ifacename.set_sensitive(False)
            self.spinbutton_network_filedescriptor.set_sensitive(False)
        elif item == "Use an already open TCP socket":
            self.entry_network_ip.set_sensitive(False)
            self.spinbutton_network_port.set_sensitive(False)
            self.entry_network_tuntapscript.set_sensitive(False)
            self.button__network_open_tuntap_file.set_sensitive(False)
            self.entry_network_ifacename.set_sensitive(False)
            self.spinbutton_network_filedescriptor.set_sensitive(True)
        elif item == "Connect to listening  TCP socket":
            self.entry_network_ip.set_sensitive(True)
            self.spinbutton_network_port.set_sensitive(True)
            self.entry_network_tuntapscript.set_sensitive(False)
            self.button__network_open_tuntap_file.set_sensitive(False)
            self.entry_network_ifacename.set_sensitive(False)
            self.spinbutton_network_filedescriptor.set_sensitive(False)
        elif item == "Create shared VLAN via UDP multicast socket":  
            self.entry_network_ip.set_sensitive(True)
            self.spinbutton_network_port.set_sensitive(True)
            self.entry_network_tuntapscript.set_sensitive(False)
            self.button__network_open_tuntap_file.set_sensitive(False)
            self.entry_network_ifacename.set_sensitive(False)
            self.spinbutton_network_filedescriptor.set_sensitive(False)
        elif item == "Use an already open UDP multicast socket":
            self.entry_network_ip.set_sensitive(False)
            self.spinbutton_network_port.set_sensitive(False)
            self.entry_network_tuntapscript.set_sensitive(False)
            self.button__network_open_tuntap_file.set_sensitive(False)
            self.entry_network_ifacename.set_sensitive(False)
            self.spinbutton_network_filedescriptor.set_sensitive(True)
        else:
            pass          
        
    def set_network_ip(self, widget):
        ip = widget.get_text()
        self.tmpnetwork.ip = ip
        
    def set_network_port(self, *widget):
        value = self.spinbutton_network_port.get_text()
        c = 0
        while value == "" and c < 100:
            c = c + 1
            return False
        valint = int(value)
        self.tmpnetwork.port = valint       
        
    def set_network_mac(self, widget):
        mac = widget.get_text()
        self.tmpnetwork.mac = mac  
        
    def set_network_vlan(self, *widget): 
        value = self.spinbutton_network_vlan.get_text()
        c = 0
        while value == "" and c < 100:
            c = c + 1
            return False
        valint = int(value)
        self.tmpnetwork.vlan = valint         
        
    def set_network_ttscript(self, widget):
        ttscript = widget.get_text()
        self.tmpnetwork.ttscript = ttscript
        #print "Set tmpnetwork tun/tap script to: " + str(self.tmpnetwork.ttscript)     
        
    def set_network_ifname(self, widget):
        ifname = widget.get_text()
        self.tmpnetwork.ifname = ifname
        #print "Set tmpnetwork interface name to: " + str(self.tmpnetwork.ifname)  
        
    def set_network_ttfiledesc(self, widget):
        value = self.spinbutton_network_filedescriptor.get_text()
        c = 0
        while value == "" and c < 100:
            c = c + 1
            return False
        valint = int(value)
        self.tmpnetwork.ttfiledesc = valint        
        
    def set_usernet_hostname(self, widget):
            network_hostname = widget.get_text()
            #print "Set Network hostname to: " + network_hostname
            self.currimage_settings.usernetwork.hostname = network_hostname 
            if self.restoreaction == False:
                self.toggle_storebuttons(True, "set_usernet_hostname")    
            
    def set_usernet_vlan(self, widget):
            value = self.spinbutton_usernet_vlan.get_text()
            c = 0
            while value == "" and c < 100:
                c = c + 1
                return False
            valint = int(value)
            self.currimage_settings.usernetwork.vlan = valint            
            if self.restoreaction == False:
                self.toggle_storebuttons(True, "set_usernet_vlan")                                                  

    def toggle_networkuse(self, widget): 
        wname = widget.get_name()
        if wname == "radiobutton_network_nonet":
            self.currimage_settings.networkmode = "nonet"
            self.frame_network_usermode.hide()
            self.frame_network_custom.hide()
            self.filechooserbutton_network_smbshare.set_sensitive(False)
            self.checkbutton_smbshare.set_sensitive(False)
        else:
            self.filechooserbutton_network_smbshare.set_sensitive(True)
            self.checkbutton_smbshare.set_sensitive(True)
           
        if wname == "radiobutton_network_usermode":
            self.currimage_settings.networkmode = "usermode"
            self.frame_network_usermode.show()
            self.frame_network_custom.hide()             
        if wname == "radiobutton_network_customsetup":
            self.currimage_settings.networkmode = "custom"
            self.frame_network_usermode.hide()
            self.frame_network_custom.show()
        
    def vpane_doubleclick(self, widget, event):
        
        if self.settingsvisible == "visible":
            if event.type == gtk.gdk._2BUTTON_PRESS:
                time.sleep(0.1)
                self.toggle_settingsview()
        else:
            if event.button == 1:
                time.sleep(0.1)
                self.toggle_settingsview()
                
    def show_settings(self, *widget):
        if self.settingsvisible != "visible":
            #gtk.threads_enter()
            self.switch_settingsview("show")
            #gtk.threads_leave()
            
    def hide_settings(self, *widget):
        if self.settingsvisible == "visible":
            #gtk.threads_enter()
            self.switch_settingsview("hide")                          
            #gtk.threads_leave()
            
    def toggle_settingsview(self, *widget):
        if self.settingsvisible == "visible":
            #gtk.threads_enter()
            self.switch_settingsview("hide")
            #gtk.threads_leave()
            
        else:
            #gtk.threads_enter()
            self.switch_settingsview("show")
            #gtk.threads_leave()
            
    def menu_toggle_settings_autohide(self, widget):
        self.settings_autohide = widget.get_active()

    def menu_toggle_settings_autoshow(self, widget):
        self.settings_autoshow = widget.get_active()

    def toggle_settings_autohide(self, status):
        self.checkbutton_autohide_settings.set_active(status)
        #print "settings_autohide toggled: " + str(status)

    def toggle_settings_autoshow(self, status):
        self.checkbutton_autoshow_settings.set_active(status)
        #print "settings_autoshow toggled: " + str(status)

    def switch_settingsview(self, showhide):
        change = False
        button_icon = gtk.Image()
        
        if showhide == "show":
            if self.item_show_settings.get_property('visible'):
                #self.vpaned_mainwindow.remove(self.vbox_bottompane)
                #self.viewport_bottompane.remove(self.vbox_bottompane)
                #self.vpaned_mainwindow.pack2(self.vbox_bottompane, True, False)
                #self.viewport_bottompane.pack(self.vbox_bottompane, True, False)
                width, height = self.mainwindow.get_size()
                newpos = height - 420
                self.vpaned_mainwindow.set_position(newpos)
                self.viewport_settings_hardware.resize_children()
                self.item_show_settings.hide()
                self.item_hide_settings.show()
    
                settingstxt = _("hide settings")
                settingstt = _("hide configuration panel for selected boot image")
    
                button_icon.set_from_icon_name("gtk-go-down", gtk.ICON_SIZE_BUTTON)
    
                self.settingsvisible = "visible"
                change = True
        if showhide == "hide":
            if self.item_hide_settings.get_property('visible'):
                #self.vpaned_mainwindow.remove(self.vbox_bottompane)
                #self.viewport_bottompane.remove(self.vbox_bottompane)
                #self.vpaned_mainwindow.pack2(self.vbox_bottompane, False, True)
                #self.viewport_bottompane.pack(self.vbox_bottompane, False, True)
                #self.vpaned_mainwindow.pack2(self.vbox_topane, False, False)
                #self.viewport_bottompane.pack(self.vbox_topane, False, False)
                self.vpaned_mainwindow.set_position(1200)
                self.item_show_settings.show()
                self.item_hide_settings.hide()
                
                settingstxt = _("show settings")
                settingstt = _("show configuration panel for selected boot image")
    
                button_icon.set_from_icon_name("gtk-go-up", gtk.ICON_SIZE_BUTTON)
    
                self.settingsvisible = "hidden"
                change = True
        if change == True:
            self.button_togglesettings.set_image(button_icon)
            self.button_togglesettings.set_label(settingstxt)
            tooltips.set_tip(self.button_togglesettings, settingstt)

    def refresh_networkcards(self, *event):
        #print "refreshing network cards"
        model = self.treeview_networkcards.get_model()
        model.clear()
        currnet = self.currimage_settings.networksetup
        if currnet != []:
            for netcard in currnet:
                ip = netcard.ip
                port = netcard.port
                vlan = netcard.vlan
                ntype = netcard.ntype
                model.append(row=(ip,port,vlan,ntype))
        self.treeview_networkcards.set_model(model) 
        #self.button_apply_imagesettings.set_sensitive(True)
        #self.button_restore_imagesettings.set_sensitive(True)
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "refresh_networkcards")                       
           
    def create_model_networkcards(self, data):
        store = gtk.ListStore(gobject.TYPE_STRING,
                          gobject.TYPE_STRING,
                          gobject.TYPE_STRING,
                          gobject.TYPE_STRING)
        for item in data:
            iter = store.append()
            ip = item[0]
            port = item[1]
            vlan = item[2]
            ntype = item[3]
            store.set(iter, 0, item[0], 1, item[1], 2, item[2], 3, item[3])
        return store    

    def add_columns_networkcards(self, treeview, model):
        renderer = gtk.CellRendererToggle()
        column0 = gtk.TreeViewColumn(_('IP'), gtk.CellRendererText(), text=0)
        column0.set_resizable(True)
        column1 = gtk.TreeViewColumn(_('Port'), gtk.CellRendererText(), text=1)
        column1.set_resizable(True)
        column2 = gtk.TreeViewColumn(_('VLAN'), gtk.CellRendererText(), text=2)
        column2.set_resizable(True)
        column3 = gtk.TreeViewColumn(_('Network Type'), gtk.CellRendererText(), text=3)
        column3.set_resizable(True)
        treeview.append_column(column0)   
        treeview.append_column(column1)        
        treeview.append_column(column2) 
        treeview.append_column(column3)  
        
## All functions for Network redirections ##

    def init_network_redirect(self, widget):
        GoOn = True     
        button = widget.get_name()
        if button == "toolbutton_network_redir_add":
            mode = "add"
            response = self.dialog_network_add_redirect_response
        if button == "toolbutton_network_redir_edit":
            mode = "edit"
            response = self.dialog_network_edit_redirect_response       
        if mode == "edit":
            try:
                model, iter = self.treeview_redirects.get_selection().get_selected()
                counter = int(model.get_string_from_iter(iter))
            except:
                print "No redirect selected"
                GoOn = False
                
        if GoOn == True:
            self.tmpredir = qml_network.Network_Redirection()
            self.tmpredir.protocol = "TCP"
            
            self.dialog_new_redirect = gtk.glade.XML(self.gladefile, "dialog_new_redirect", APP)
            self.dialog_new_redirect_widget = self.dialog_new_redirect.get_widget("dialog_new_redirect")
            ## widgets of the new image dialog
            self.radiobutton_redirect_TCP = self.dialog_new_redirect.get_widget("radiobutton_redirect_TCP")
            self.radiobutton_redirect_UDP = self.dialog_new_redirect.get_widget("radiobutton_redirect_UDP")
            self.spinbutton_redirect_sport = self.dialog_new_redirect.get_widget("spinbutton_redirect_sport")            
            self.entry_redirect_gIP = self.dialog_new_redirect.get_widget("entry_redirect_gIP")
            self.spinbutton_redirect_dport = self.dialog_new_redirect.get_widget("spinbutton_redirect_dport")        
            self.button_redirect_add = self.dialog_new_redirect.get_widget("button_redirect_add")
            dic = {"on_radiobutton_redirect_TCP_toggled" : self.set_redir_prot
            , "on_radiobutton_redirect_UDP_toggled" : self.set_redir_prot
            , "on_spinbutton_redirect_sport_changed" : self.set_redir_sport
            , "on_entry_redirect_gIP_changed" : self.set_redir_ip
            , "on_spinbutton_redirect_dport_changed" : self.set_redir_dport
            , "on_dialog_new_redirect_response" : response
            }
            self.dialog_new_redirect.signal_autoconnect(dic)
            if mode == "edit":
                self.button_redirect_add.set_label("Apply")
                image = gtk.Image()
                image.set_from_stock(gtk.STOCK_APPLY, gtk.ICON_SIZE_BUTTON)
                self.button_redirect_add.set_image(image)
                self.dialog_new_redirect_widget.set_title("Edit selected redirection")
                self.network_editredir = self.currimage_settings.networkredirs[counter]
                self.tmpredir = self.network_editredir
                
                if self.network_editredir.protocol == "tcp":
                    self.radiobutton_redirect_TCP.set_active(True)
                if self.network_editredir.protocol == "udp":
                    self.radiobutton_redirect_UDP.set_active(True)                    
                self.spinbutton_redirect_sport.set_value(int(self.network_editredir.sport))
                self.entry_redirect_gIP.set_text(str(self.network_editredir.ip))
                self.spinbutton_redirect_dport.set_value(int(self.network_editredir.dport))
                   
     
    def dialog_network_add_redirect_response(self, widget, response, *entry):    
        if response == gtk.RESPONSE_CLOSE:
            self.tmpredir = []
            widget.destroy () 
        if response == gtk.RESPONSE_APPLY:
            self.currimage_settings.networkredirs.append(copy(self.tmpredir))
            self.refresh_redirects()
            self.toggle_storebuttons(True, "dialog_network_add_redirect_response")
            
    def dialog_network_edit_redirect_response(self, widget, response, *entry):    
        if response == gtk.RESPONSE_CLOSE:
            self.tmpredir = []
            widget.destroy ()
        if response == gtk.RESPONSE_APPLY:
            self.network_editredir = copy(self.tmpredir)
            self.refresh_redirects()
            self.toggle_storebuttons(True, "dialog_network_edit_redirect_response")
            
    def refresh_redirects(self, *event):
        model = self.treeview_redirects.get_model()
        model.clear()
        curredirs = self.currimage_settings.networkredirs
        if self.currimage_settings.networkredirs != []:
            for redir in curredirs:
                protocol = redir.protocol
                sport = redir.sport
                ip = redir.ip
                dport = redir.dport
                model.append(row=(protocol,sport,ip,dport))

    def create_model_redirects(self, data):
        store = gtk.ListStore(gobject.TYPE_STRING,
                          gobject.TYPE_STRING,
                          gobject.TYPE_STRING,
                          gobject.TYPE_STRING)
        for item in data:
            iter = store.append()
            protocol = item[0]
            sport = item[1]
            ip = item[2]
            dport = item[3]

            store.set(iter, 0, item[0], 1, item[1], 2, item[2], 3, item[3])
        return store    

    def add_columns_redirects(self, treeview, model):
    
        renderer = gtk.CellRendererToggle()
        column0 = gtk.TreeViewColumn(_('Protocol'), gtk.CellRendererText(), text=0)
        column0.set_resizable(True)
        column1 = gtk.TreeViewColumn(_('Source port'), gtk.CellRendererText(), text=1)
        column1.set_resizable(True)
        column2 = gtk.TreeViewColumn(_('Guest IP'), gtk.CellRendererText(), text=2)
        column2.set_resizable(True)
        column3 = gtk.TreeViewColumn(_('Dest. port'), gtk.CellRendererText(), text=3)
        column3.set_resizable(True)
        treeview.append_column(column0)   
        treeview.append_column(column1)        
        treeview.append_column(column2) 
        treeview.append_column(column3)  

    def set_redir_prot(self, widget):
        wname = widget.get_name()
        
        if wname == "radiobutton_redirect_TCP":
            self.tmpredir.protocol = "tcp"
        elif wname == "radiobutton_redirect_UDP":
            self.tmpredir.protocol = "udp"

    def set_redir_sport(self, *widget):
        value = self.spinbutton_redirect_sport.get_text()
        c = 0
        while value == "" and c < 100:
            c = c + 1
            return False
        valint = int(value)
        self.tmpredir.sport = valint           
        
    def set_redir_ip(self, widget):
        ip = widget.get_text()
        self.tmpredir.ip = ip
        #print "Set tmpredir ip to: " + str(self.tmpredir.ip)          
        
    def set_redir_dport(self, *widget):
        port = int(self.spinbutton_redirect_dport.get_value())
        #print "changed dport to: " + str(port)
        self.tmpredir.dport = port 
        value = self.spinbutton_redirect_dport.get_text()
        c = 0
        while value == "" and c < 100:
            c = c + 1
            return False
        valint = int(value)
        self.tmpredir.dport = valint           
        
    def remove_redirect(self, widget):
        try:
            model, iter = self.treeview_redirects.get_selection().get_selected()
            protocol = model.get_value(iter, 0)
            sport = model.get_value(iter, 1)
            ip = model.get_value(iter, 2)
            dport = model.get_value(iter, 3)
            counter = int(model.get_string_from_iter(iter))
            del self.currimage_settings.networkredirs[counter]
            self.refresh_redirects()  
            self.toggle_storebuttons(True, "remove_redirect")
        except:
            print "No redirection selected"       
            
    def set_smbsettings(self, widget):    
        if widget == self.checkbutton_smbshare:
            self.currimage_settings.smbsettings.usesmb = widget.get_active()
        elif widget == self.filechooserbutton_network_smbshare:
            self.currimage_settings.smbsettings.smbpath = widget.get_filename()  
        if self.restoreaction == False:
            self.toggle_storebuttons(True, "set_smbsettings")
   
    def run_jobmanager(self, *widget):
        selected_job_path = self.selected_job[0]
        selected_job_pid = self.selected_job[1]
        selected_job_pty = self.selected_job[2]
        selected_job_displaymode = self.selected_job[3]
        selected_job_runstat = self.selected_job[4]
        selected_job_name = selected_job_path
        
        print "JOB: " + str(self.selected_job)
        
        try:
            if self.image_settings[selected_job_path]:
               image_settings = self.image_settings[selected_job_path] 
        except:
            image_settings = None

        if len(self.selected_job) == 6:
            selected_job_name = self.selected_job[5]        
        if self.jobmanagers.has_key(selected_job_pid):
            #print "Jobmanager for " + str(selected_job_pid) + " only hidden, no new instance will be raised"
            self.jobmanagers[selected_job_pid].dialog_jobmonitor.show()
            if tray != None:
                tray.traymenu_item[selected_job_pid].set_active(True)
        else:
            #print "Jobmanager for " + str(selected_job_pid) + " not open, new instance will be raised"
            self.jobmanagers[selected_job_pid] = qml_controlmon.ControlMonitor(self.selected_job, image_settings, tray, self.joblist, self.close_jobmanager)
            #gtk.threads_enter()
            self.jobmanagers[selected_job_pid].start()
            #gtk.threads_leave()
            
            if tray != None:
                print "job: " + str(self.selected_job)
                tray.create_trayentry_jobmanager(self.selected_job)
        
    def close_jobmanager(self, pid):
        if tray != None:
            tray.remove_trayentry_jobmanager(pid)
        del self.jobmanagers[pid]      
        
    def check_firstrun(self):
        if os.path.exists(self.homedir):
            #print "homedir '" + self.homedir + "' allready exists"
            pass
        else:
            print "homedir '" + self.homedir + "' will be created"
            os.mkdir(self.homedir)
            
        if os.path.exists(self.configdir):
            #print "configdir '" + self.configdir + "' allready exists"
            pass
        else:
            print "configdir '" + self.configdir + "' will be created"
            os.mkdir(self.configdir)            
                    
        if os.path.exists(self.icondirs[1]):
            #print "icondir '" + self.icondirs[1] + "' allready exists"
            pass
        else:
            print "icondir '" + self.icondirs[1] + "' will be created"
            os.mkdir(self.icondirs[1])
            icons=os.listdir(self.icondirs[0])
            for iconam in icons:
                #print str(iconam)
                srcicon = self.icondirs[0] + "/" + iconam
                dsticon = self.icondirs[1] + "/" + iconam
                #print srcicon + " to"
                #print dsticon
                if os.path.isfile(srcicon):
                    shutil.copy(srcicon, dsticon)
                    
    def open_website(self, widget, event=None, url="http://qemulator.createweb.de"):
        browser = Webbrowser()  
        browser.url = url
        browser.target = "new window"
        browser.start()

class QemuConfig:  
    def __init__ (self, image=None):
        self.currentimage = image
        
        self.emusystem = ""
        self.machine = ""
        self.kqemu_param = ""
        self.bootmedium = "cdrom"
        self.boot_driveletter = "d"
        self.cdrmedium = None
        self.cdrimage = None
        self.cdrmode = "nocd"
        self.hda_image = None  
        self.hdb_image = None
        self.hdc_image = None
        self.hdd_image = None
        self.fda_image = None
        self.fdb_image = None
        self.soundhw = ""
        self.vgamode = ""
        self.usevnc = False
        self.vncdisplay = 1
        self.useusb = False
        self.novga = False
        self.fullscreenmode = False
        self.dlinuxboot = False
        self.kernelimage = None
        self.kernelcmd = ""
        self.initrd = None
        self.snapshotmode = False
        self.localtimemode = False
        self.memory = 128
        self.customoptions = ""
        self.bootfromcd = False
        self.networksetup = []
        self.networkmode = "usermode"
        self.networkredirs = []
        self.network_hostname = "emu"
        self.usernetwork = qml_network.Network_Usernet()
        self.smbsettings = qml_network.Network_SMB()
        
#class Joblist(Thread):
class Joblist:         
    def __init__(self, treeview, joblist_data = []):
        #Thread.__init__(self)
        self.configdir = configdir
        self.treeview = treeview
        self.joblist_data = []
        self.jobmanagers = {}
        
    def run(self):
        pass
                  
    def create_model_joblist(self, data):
        store = gtk.ListStore(gobject.TYPE_STRING,
                              gobject.TYPE_STRING,
                              gobject.TYPE_STRING,
                              gobject.TYPE_STRING,
                              gobject.TYPE_STRING)
        for item in data:
            if item != 0:
                iter = store.append()
                pid = item[0]
                pts = item[1]
                ipath = item[2]
                displaymode = item[3]
                runstat = item[4]
                store.set(iter, 0, ipath, 1, pid, 2, pts, 3, displaymode, 4, runstat)
        return store  
   
    def add_columns_joblist(self, treeview, model):
    
        renderer = gtk.CellRendererToggle()
        column0 = gtk.TreeViewColumn(_('Boot image'), gtk.CellRendererText(), text=0)
        column0.set_resizable(True)
        column1 = gtk.TreeViewColumn(_('PID'), gtk.CellRendererText(), text=1)
        column1.set_resizable(True)
        column2 = gtk.TreeViewColumn(_('PTS'), gtk.CellRendererText(), text=2)
        column2.set_resizable(True)
        column3 = gtk.TreeViewColumn(_('Display'), gtk.CellRendererText(), text=3)
        column3.set_resizable(True)
        column4 = gtk.TreeViewColumn(_('running'), gtk.CellRendererText(), text=4)
        column4.set_resizable(True)
        
        treeview.append_column(column0)  
        treeview.append_column(column1) 
        treeview.append_column(column2)
        treeview.append_column(column3)      
        treeview.append_column(column4)
        
    def buildtree_joblist(self, *widget):    
        data = self.joblist_data
        model = self.create_model_joblist(data) 
        self.add_columns_joblist(self.treeview, model)
        self.treeview.set_model(model)
        
    def refresh_joblist(self, *widget):    
            data = self.joblist_data
            model1 = self.treeview.get_model()
            try:
                model1.clear()
                #print "model joblist cleared"
            except:
                #print "model joblist empty, not cleared"
                pass
            model = self.create_model_joblist(data)
            setmodel = self.treeview.set_model(model)
            time.sleep(0.01)
            self.store_joblist()
            
    def pause_job(self, widget, job): 
        pts = job[2] 
        ptsin = qml_ptsmanager.PtsMan(pts, "stop").start()
        for jobs in self.joblist_data:
            if jobs != []:
                print "pause_job check: " + str(jobs[1])
                if jobs[1] == pts:
                    print "pause_job found: " + str(jobs[1]) + " == " + str(pts)
                    jindex = self.joblist_data.index([jobs[0], jobs[1], jobs[2], jobs[3], jobs[4]])
                    self.joblist_data[jindex] = [jobs[0], jobs[1], jobs[2], jobs[3], "0"]
                    #print "jobstop: " + str(self.joblist_data[jc][2])
        if self.refresh_joblist != None:
            self.refresh_joblist()
        print "job paused"
         
    def cont_job(self, widget, job):
        pts = job[2]
        ptsin = qml_ptsmanager.PtsMan(pts, "cont").start()
        for jobs in self.joblist_data:
            if jobs != []:
                print "cont_job check: " + str(jobs[1])
                if jobs[1] == pts:
                    print "cont_job found: " + str(jobs[1]) + " == " + str(pts)
                    jindex = self.joblist_data.index([jobs[0], jobs[1], jobs[2], jobs[3], jobs[4]])
                    self.joblist_data[jindex] = [jobs[0], jobs[1], jobs[2], jobs[3], "1"]
        if self.refresh_joblist != None:
            self.refresh_joblist()    
        print "job continued"                
    
    def reset_job(self, widget, job):
        pts = job[2]
        ptsin = qml_ptsmanager.PtsMan(pts, "system_reset").start() 
        print "reset done"    
        
    def stop_job(self, widget, job):        
        pts = job[2]
        ptsin = qml_ptsmanager.PtsMan(pts, "quit").start()        
        
    def search_job(self, pid, pts=None):
        index = None
        if not self.joblist_data or self.joblist_data == []:
            index = None
        else:
            for job in self.joblist_data:
                #print "searching for: " + str(job)
                if pts == None:
                    if job[0] == pid:
                        #print "found match: " + str(job[0]) + " = " + str(pid)
                        index = self.joblist_data.index(job)
                        break
                else:
                    if job[0] == pid and job[1] == pts:
                        #print "found match: " + str(job[0]) + " = " + str(pid) + " " + str(job[1]) + "=" + str(pts)
                        index = self.joblist_data.index(job)
                        break
        return index
    
    def add_job(self, job):
        #print 'widget: ' + str(widget)
        #print 'response: ' + str(response)
        pid = job[0]
        pts = job[1]
        ipath = job[2]
        displaymode = job[3]
        pts = str(pts)
        runstat = 1
        self.joblist_data.append(job)
        self.refresh_joblist()
        #print str(self.joblist_data)  
 
    def remove_job(self, pid, pts):
        entry_pts = pts
        entryindex = self.search_job(pid, pts)

        if entryindex != "None":
            del self.joblist_data[entryindex]
        self.refresh_joblist()
        
    def load_joblist(self):
         self.joblist_data = getfromfile("joblist")
         #print "loaded joblist data: " + str(self.joblist_data)
         if self.joblist_data == {}:
             self.joblist_data = []
         self.refresh_joblist()
         #return self.joblist_data
         
    def store_joblist(self):
        if self.joblist_data == [] or self.joblist_data == "":
            if os.path.exists(configdir + "/joblist"):
                remjoblist = os.remove(configdir + "/joblist")
        else:
            storetofile(self.joblist_data, "joblist")
        
    def remove_job2(self, job):
        pid = job[0]
        pts = job[1]
        image = job[2]
        entryindex = self.search_job(pid, pts)
        if entryindex != None:
            del self.joblist_data[entryindex]

            if len(qemulator.jobmanagers) != 0: 
                kpid = str(pid)
                try:
                    qemulator.jobmanagers[kpid].jobmonitor_close()
                except:
                    pass
        else:
            pass
        time.sleep(0.1)
        self.refresh_joblist()

class QemuGetPts ( Thread ):
    def __init__ (self, pid, childout, childerr):
        Thread.__init__(self)
        self.childout = childout
        self.childerr = childerr
        self.errormessage = ""
        self.pid = pid
    def run(self):
        #print "QemuOutThread aufgerufen"
        errormessage = ""
        checkpid = os.popen("ps -e | grep " + str(self.pid)).readlines()
        print "daemon id checkstat: " + str(checkpid)
        if checkpid != []:
            while 1:
                linerr = self.childerr.readline() 
                errormessage = errormessage + linerr
                if linerr == "": break
                #print linerr
                m = re.search("/dev/pts/\d+", linerr)
                if m != None:
                    self.currentpts = m.group()
                    self.errormessage = errormessage
                    break
        self.errormessage = errormessage
        #return self.errormessage
        
class QemuWatcher ( Thread ):

    def __init__ (self, rem_func=None):
        #time.sleep(2)
        Thread.__init__(self)
        self.rem_func = rem_func
        self.joblist = []
        self.status = "stopped"

    def run(self):
        #self.status = "running"
        print "watcher daemon started"
        while self.joblist != []:
            self.status = "running"
            for job in self.joblist:
                checkpid = ""
                pid = job[0]
                pts = job[1]
                image = job[2]
                checkpid = os.popen("ps -e | grep " + str(pid)).readlines()
                #print "daemon id checkstat: " + str(checkpid)
                if os.path.exists(pts) and checkpid != "":
                   #print "daemon pts found: " + str(pts)
                   pass              
                else: 
                    print pts + " not found - removed from daemon list"
                    delindex = self.joblist.index(job)
                    
                    del self.joblist[delindex]
                    if self.rem_func != None:
                        gtk.threads_enter()
                        dofunc = self.rem_func(job)
                        gtk.threads_leave()
                    #del self.joblist[delindex]
                    break
            if qemulator.dorun != True:
                print "mainapp closed, exit watcher daemon"
                break 
            time.sleep(3)
        self.status = "stopped"
        print "no more jobs running - daemon QUIT"
    
class Bookmark:        
    def __init__(self):
        #print "bookmarks caller: " + str(caller)
        self.name = "new machine"
        self.icondir = icondirs[0]
        self.icon = self.icondir + "/default.png"    
        self.path = ""
        self.haveconfig = False
        #self.pixmapdir = qemulator.pixmapdir
        self.pixmapdir = pixmapdir
        #print "bookmarks pixmapdir " + pixmapdir
        #print "bookmarks self.pixmapdir " + self.pixmapdir
    def get_modeldata(self):
        self.pixmapdir = pixmapdir
        icon = None
        try:
            icon = gtk.gdk.pixbuf_new_from_file(self.icon)
        except:
            icon = None
        if self.haveconfig == True:
            config_icon = gtk.gdk.pixbuf_new_from_file(self.pixmapdir + '/gtk-yes.png')
        else:
            config_icon = gtk.gdk.pixbuf_new_from_file(self.pixmapdir + '/gtk-no.png')
        modeldata = [icon, self.name, self.path, config_icon]
        return modeldata
    def check_haveconfig(self, bootimage):
        haveconfig = False
        try:
            if self.image_settings[bootimage]:
                #print "Config for " + str(bootimage) + " found"
                haveconfig = True
            else:
                #print "No config for " + str(bootimage) + " found"
                haveconfig = False
        except:       
            #print "No config for " + str(bootimage) + " found"
            haveconfig = False
        return haveconfig
                                                
class Webbrowser ( Thread ):
    def __init__ (self):
        Thread.__init__(self)
        self.url = "http://qemulator.createweb.de"
        self.target = "new window"

    def run(self):
        self.open_website(self.url, self.target)  

    def open_website(self, url, target):
        if target == "new window":
            newin = 1
        else:
            newin = 0
        runweb = webbrowser.open(self.url, new=newin)
         
class TrayMenuitem(gtk.CheckMenuItem):
    def init(self, label=None, use_underline=True):
        self.label = label
        self.use_underline = use_underline
    
        self.job = None
        if self.job != None:
            self.path = job[0]
            self.pid = job[1]
            self.pts = job[2]
            
#class Dialog ( Thread ):
class Dialog:
    def __init__ (self, type, text, title=None, returnfunc=None, return_args=[]):
        #Thread.__init__(self)
        self.type = type
        self.text = text
        self.title = title
        self. returnfunc = returnfunc
        self.return_args = ",".join(return_args)
        self.gladefile = libdir + "/qemulator.glade" 
        #self.run()  
        self.init_dialog() 

    #def run(self): 
    #    self.init_dialog()
            
    def init_dialog(self):
        self.wTree = gtk.glade.XML(self.gladefile, "dialog_universal", APP)
        self.dialog = self.wTree.get_widget("dialog_universal")
        self.dialog.set_keep_above(True)
        self.label_messagetext = self.wTree.get_widget("label_messagetext")
        self.image_messageitem = self.wTree.get_widget("image_messageitem")
        
        self.applybutton = self.wTree.get_widget("applybutton")
        self.cancelbutton = self.wTree.get_widget("cancelbutton")
        self.okbutton = self.wTree.get_widget("okbutton")
        self.closebutton = self.wTree.get_widget("closebutton")
        dic = {"on_dialog_universal_response" : self.dialog_response,        
        }
        self.wTree.signal_autoconnect(dic)
        if self.type == "info":
            self.applybutton.hide()
            self.cancelbutton.hide()
            self.okbutton.hide()
            self.image_messageitem.set_from_stock(gtk.STOCK_INFO, gtk.ICON_SIZE_DIALOG)
        if self.type == "message":
            self.applybutton.hide()
            self.cancelbutton.hide()
            self.closebutton.hide()
            self.image_messageitem.set_from_stock(gtk.STOCK_DIALOG_INFO, gtk.ICON_SIZE_DIALOG)        
        elif self.type == "warning":
            self.applybutton.hide()
            self.cancelbutton.hide()
            self.closebutton.hide()
            self.image_messageitem.set_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_DIALOG)
        elif self.type == "error":
            self.applybutton.hide()
            self.cancelbutton.hide()
            self.closebutton.hide()
            self.image_messageitem.set_from_stock(gtk.STOCK_DIALOG_ERROR, gtk.ICON_SIZE_DIALOG)            
        elif self.type == "request":
            self.applybutton.hide()
            self.closebutton.hide()
            self.image_messageitem.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)
        elif self.type == "apply1":
            self.closebutton.hide() 
            self.image_messageitem.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)            
        elif self.type == "apply2":
            self.cancelbutton.hide()
            self.image_messageitem.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)

        if self.title != None:
            self.dialog.set_title(self.title)
        self.label_messagetext.set_markup(self.text)
        self.label_messagetext.queue_resize()
        
    def dialog_response(self, widget, response, *entry):      
        if response == gtk.RESPONSE_APPLY or response == gtk.RESPONSE_OK:
            if self. returnfunc != None:
                self. returnfunc(self.return_args)
                time.sleep(0.1) 
            if response == gtk.RESPONSE_OK:
                widget.destroy ()
            return True        
        elif response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_CLOSE:
            widget.destroy ()
            return False                                   

class Tray ( Thread ):

    def __init__ (self):
        Thread.__init__(self) 
        self.traymenu_item = {}
        self.gladefile = libdir + "/qemulator.glade"
          
    def run(self):   
        self.init_tray()

    def init_tray(self):
        gtk.threads_enter()
        self.wTree_tray_menu = gtk.glade.XML(self.gladefile,"tray_menu", APP)
        self.tray_menu = self.wTree_tray_menu.get_widget("tray_menu")
        self.traymen_item_exit = self.wTree_tray_menu.get_widget("item_exit")
        
        self.dic_menu_tray = {"on_item_exit_activate" : qemulator.exit_app_event      
        }
        self.wTree_tray_menu.signal_autoconnect(self.dic_menu_tray) 
        self.tray_icon = trayicon.TrayIcon("Qemulator")
        eventbox = gtk.EventBox()
        self.tray_icon.add(eventbox)

        eventbox.connect("button_press_event", self.on_tray_button_press_event)
        eventbox.connect("button_release_event", self.on_tray_button_release_event)

        icon_file = pixmapdir + "/trayicon.png"
        image = gtk.Image()
        image.set_from_file(icon_file)
        image.set_pixel_size(gtk.ICON_SIZE_SMALL_TOOLBAR)
        eventbox.add(image)
        
        self.tray_menu.set_title("Qemulator")
        tearoffitem = gtk.TearoffMenuItem()
        self.tray_menu.prepend(tearoffitem)
        self.tray_menu.append(gtk.SeparatorMenuItem()) 
        
        self.tray_menu.show_all()
        self.tray_icon.show_all()       
        
        gtk.threads_leave()
        
    def popup_menu(self, args):
        self.tray_menu.show_all()
        self.tray_icon.show_all()
        self.tray_menu.popup(None, None, None, 0, 0)
        return 1
    
    def resize_popup(self, widget=None, *event):
        print "resizing popup"

    def on_tray_button_press_event(self, *args):
        event = args[1]
        if args[1].button == 1:
            if qemulator.mainwindow.get_property('visible'):
                qemulator.mainwindow.hide()
            else:
                qemulator.mainwindow.present()

    def on_tray_button_release_event(self, *args):
        #print "args: " + str(args) + " --type " + str(args[1].type)

        if args[1].button == 3:
            self.popup_menu(args)               
            
    def create_trayentry_jobmanager(self, job):
        selected_job_path = job[0]
        selected_job_name = selected_job_path
        selected_job_pid = job[1]
        if len(job) == 6:
            selected_job_name = job[5]
        jobname = "Monitor for: " + str(selected_job_name)

        self.traymenu_item[selected_job_pid] = TrayMenuitem(jobname, False)
        self.traymenu_item[selected_job_pid].set_active(True) 
        self.traymenu_item[selected_job_pid].job = job
        self.tray_menu.append(self.traymenu_item[selected_job_pid])
        self.traymenu_item[selected_job_pid].connect("button_press_event", self.on_tray_entry_jobman)
        self.tray_menu.show_all()
        
    def on_tray_entry_jobman(self, *args):
        #print "args: " + str(args)
        widget = args[0]
        wname = widget.get_name()
        job = widget.job
        #print "job: " + str(job)
        selected_job_pid = job[1]
        jobmonitor = qemulator.jobmanagers[selected_job_pid]
        if jobmonitor.dialog_jobmonitor.get_property('visible'):
            jobmonitor.to_tray(widget)
        else:
            jobmonitor.restore_from_tray(widget)    
        
    def remove_trayentry_jobmanager(self, pid):
        #print "Item " + str(self.traymenu_item[pid]) + " destroyed"
        self.traymenu_item[pid].destroy()
        self.tray_menu.show_all() 
        
def load_settings(): 
    #print "load_settings called"
    settings = {}
    settings["cdromdrives"] = [("CD-rom", "/dev/cdrom"),]
    settings["qemuapps"] = [("x86", "qemu"), 
    ("x86_64", "qemu-system-x86_64"),
    ("ppc", "qemu-system-ppc"),
    ("arm", "qemu-system-arm"), 
    ("mips", "qemu-system-mips"),
    ("mipsel", "qemu-system-mipsel"),
    ("sparc", "qemu-system-sparc"),]
    settings["monitor_hotkeys"] = ["ctrl-alt-f1", 
                                   "ctrl-alt-f2", 
                                   "ctrl-alt-f3",
                                   "ctrl-alt-f4",
                                   "ctrl-alt-f5",
                                   "ctrl-alt-f6",
                                   "ctrl-alt-f7",
                                   "ctrl-alt-delete"]
    settings["usetray"] = True
    settings["closetotray"] = False
    settings["controlmon_totray"] = False
    settings["controlmon_ministart"] = False
    settings["controlmon_ontop"] = False
    
    settings["imagespath"] = os.path.expanduser('~/')
    settings["default_iconsize"] = 32
    settings["settings_autoshow"] = True
    settings["settings_autohide"] = True
    settings["helperapps_vncviewer"] = "vncviewer"
    settings["helperapps_vncpassfile"] = ""    
    settings["helperapps_imageviewer"] = "eog"
    settings["helperapps_audioplayer"] = "xmms"     

    outdata = getfromfile("mainsettings")
        
    if outdata  != {} and outdata != "":
        settings.update(outdata)
        #print "loaded settings from file"       
    return settings                       
                             
def filter_devs(settings):
    filter = gtk.FileFilter()
    for volume in settings["cdromdrives"]:
        pattern = volume[1].replace ( '/dev/', '' ) 
        filter.add_pattern(pattern)
        filter.set_name(_("Only configured devices"))
    return filter

def filter_shall(settings):
    filter = gtk.FileFilter()
    pattern = '*'
    filter.add_pattern(pattern)
    filter.set_name(_("All Files"))
    return filter    

def reload_settings():
    settings = {}
    settings = load_settings()
    print "Get settings from reload_settings function" 

if __name__ == "__main__":
      
    settings = {}
    settings = load_settings()       

    splashscreen = Splashscreen()
    splashscreen.start()
    time.sleep(1.0)
 
    
    qemulator = Qemulator(settings)
    #gobject.idle_add(qemulator.start)
    qemulator.start()    
    #time.sleep(0.1)
    tray = None
    if nognomextras != True and settings["usetray"] == True:
        tray = Tray()
        #gobject.idle_add(tray.start)
        tray.start()
    gtk.threads_enter()
    gtk.main()
    gtk.threads_leave()
